@@ -5,8 +5,8 @@ C.12 Line and Page Breaking (p212)
 
 """
 
-from plasTeX import Command, Environment
-from plasTeX.Logging import getLogger
+from plasTeX import Command
+
 
 #
 # C.12.1
