@@ -5,10 +5,10 @@ C.7 Mathematical Formulas (p187)
 
 """
 
-from Arrays import Array
+from plasTeX.Base.LaTeX.Arrays import Array
 from plasTeX import Command, Environment, sourceChildren
 from plasTeX import DimenCommand, GlueCommand
-from plasTeX.Logging import getLogger
+
 
 #
 # C.7.1
@@ -19,33 +19,43 @@ from plasTeX.Logging import getLogger
 
 class ThinSpace(Command):
     macroName = '.'
-    unicode = u'\u2009'
+    str = '\u2009'
 
 class NegativeThinSpace(Command):
     macroName = '!'
 
 class MediumSpace(Command):
     macroName = ':'
-    unicode = u'\u8196'
+    str = '\u8196'
 
 class ThickSpace(Command):
     macroName = ';'
-    unicode = u'\u8194'
+    str = '\u8194'
 
 class ThinSpace_(Command):
     macroName = '/'
-    unicode = u'\u2009'
+    str = '\u2009'
 
 class MathEnvironment(Environment):
     mathMode = True
 
+class MathEnvironmentPre(MathEnvironment):
+    """
+    A math environment whose source property keeps the begin and end markup.
+    """
+    @property
+    def source(self):
+        return u"\\begin{{{0}}}{1}\\end{{{0}}}".format(
+                self.tagName,
+                sourceChildren(self))
+
 # Need \newcommand\({\begin{math}} and \newcommand\){\end{math}}
 
 class math(MathEnvironment):
     @property
     def source(self):
         if self.hasChildNodes():
-            return '$%s$' % sourceChildren(self)
+            return u'$%s$' % sourceChildren(self)
         return '$'
 
 class displaymath(MathEnvironment):
@@ -138,17 +148,19 @@ class eqnarray(EqnarrayStar):
             return res
         res[1].ref = self.ref
         return res
-
 class nonumber(Command):
 
     def invoke(self, tex):
         self.ownerDocument.context.counters['equation'].addtocounter(-1)
 
     def digest(self, tokens):
-        row = self.parentNode
-        while not isinstance(row, Array.ArrayRow):
-            row = row.parentNode
-        row.ref = None
+        try:
+            row = self.parentNode
+            while not isinstance(row, Array.ArrayRow):
+                row = row.parentNode
+            row.ref = None
+        except AttributeError as e:
+            print('problem encountered %s' % e)
 
 class notag(nonumber):
     pass
@@ -194,7 +206,7 @@ class sqrt(Command):
     args = '[ n ] self'
 
 class ldots(Command):
-    unicode = u'\u2026'
+    str = '\u2026'
 
 class cdots(Command):
     pass
@@ -217,92 +229,92 @@ class MathSymbol(Command):
     pass
 
 # Lowercase
-class alpha(MathSymbol): unicode = unichr(945)
-class beta(MathSymbol): unicode = unichr(946)
-class gamma(MathSymbol): unicode = unichr(947)
-class delta(MathSymbol): unicode = unichr(948)
-class epsilon(MathSymbol): unicode = unichr(949)
-class varepsilon(MathSymbol): unicode = unichr(949)
-class zeta(MathSymbol): unicode = unichr(950)
-class eta(MathSymbol): unicode = unichr(951)
-class theta(MathSymbol): unicode = unichr(952)
-class vartheta(MathSymbol): unicode = unichr(977)
-class iota(MathSymbol): unicode = unichr(953)
-class kappa(MathSymbol): unicode = unichr(954)
+class alpha(MathSymbol): str = chr(945)
+class beta(MathSymbol): str = chr(946)
+class gamma(MathSymbol): str = chr(947)
+class delta(MathSymbol): str = chr(948)
+class epsilon(MathSymbol): str = chr(949)
+class varepsilon(MathSymbol): str = chr(949)
+class zeta(MathSymbol): str = chr(950)
+class eta(MathSymbol): str = chr(951)
+class theta(MathSymbol): str = chr(952)
+class vartheta(MathSymbol): str = chr(977)
+class iota(MathSymbol): str = chr(953)
+class kappa(MathSymbol): str = chr(954)
 class GreekLamda(MathSymbol):
     macroName = 'lambda'
-    unicode = unichr(955)
-class mu(MathSymbol): unicode = unichr(956)
-class nu(MathSymbol): unicode = unichr(957)
-class xi(MathSymbol): unicode = unichr(958)
-class pi(MathSymbol): unicode = unichr(960)
-class varpi(MathSymbol): unicode = unichr(982)
-class rho(MathSymbol): unicode = unichr(961)
-class varrho(MathSymbol): unicode = unichr(1009)
-class sigma(MathSymbol): unicode = unichr(963)
-class varsigma(MathSymbol): unicode = unichr(962)
-class tau(MathSymbol): unicode = unichr(964)
-class upsilon(MathSymbol): unicode = unichr(965)
-class phi(MathSymbol): unicode = unichr(966)
-class varphi(MathSymbol): unicode = unichr(981)
-class chi(MathSymbol): unicode = unichr(967)
-class psi(MathSymbol): unicode = unichr(968)
-class omega(MathSymbol): unicode = unichr(969)
+    str = chr(955)
+class mu(MathSymbol): str = chr(956)
+class nu(MathSymbol): str = chr(957)
+class xi(MathSymbol): str = chr(958)
+class pi(MathSymbol): str = chr(960)
+class varpi(MathSymbol): str = chr(982)
+class rho(MathSymbol): str = chr(961)
+class varrho(MathSymbol): str = chr(1009)
+class sigma(MathSymbol): str = chr(963)
+class varsigma(MathSymbol): str = chr(962)
+class tau(MathSymbol): str = chr(964)
+class upsilon(MathSymbol): str = chr(965)
+class phi(MathSymbol): str = chr(966)
+class varphi(MathSymbol): str = chr(981)
+class chi(MathSymbol): str = chr(967)
+class psi(MathSymbol): str = chr(968)
+class omega(MathSymbol): str = chr(969)
 
 # Uppercase
-class Gamma(MathSymbol): unicode = unichr(915)
-class Delta(MathSymbol): unicode = unichr(916)
-class Theta(MathSymbol): unicode = unichr(920)
-class Lambda(MathSymbol): unicode = unichr(923)
-class Xi(MathSymbol): unicode = unichr(926)
-class Pi(MathSymbol): unicode = unichr(928)
-class Sigma(MathSymbol): unicode = unichr(931)
-class Upsilon(MathSymbol): unicode = unichr(978)
-class Phi(MathSymbol): unicode = unichr(934)
-class Psi(MathSymbol): unicode = unichr(936)
-class Omega(MathSymbol): unicode = unichr(8486)
+class Gamma(MathSymbol): str = chr(915)
+class Delta(MathSymbol): str = chr(916)
+class Theta(MathSymbol): str = chr(920)
+class Lambda(MathSymbol): str = chr(923)
+class Xi(MathSymbol): str = chr(926)
+class Pi(MathSymbol): str = chr(928)
+class Sigma(MathSymbol): str = chr(931)
+class Upsilon(MathSymbol): str = chr(978)
+class Phi(MathSymbol): str = chr(934)
+class Psi(MathSymbol): str = chr(936)
+class Omega(MathSymbol): str = chr(8486)
 
 
 #
 # Table 3.4: Binary Operation Symbols
 #
 
-class pm(MathSymbol): unicode = unichr(177)
-class mp(MathSymbol): unicode = unichr(8723)
-class times(MathSymbol): unicode = unichr(215)
-class div(MathSymbol): unicode = unichr(247)
-class ast(MathSymbol): unicode = unichr(42)
-class star(MathSymbol): unicode = unichr(8902)
-class circ(MathSymbol): unicode = unichr(9675)
-class bullet(MathSymbol): unicode = unichr(8226)
-class cdot(MathSymbol): unicode = unichr(183)
-class cap(MathSymbol): unicode = unichr(8745)
-class cup(MathSymbol): unicode = unichr(8746)
-class uplus(MathSymbol): unicode = unichr(8846)
-class sqcap(MathSymbol): unicode = unichr(8851)
-class sqcup(MathSymbol): unicode = unichr(8852)
-class vee(MathSymbol): unicode = unichr(8744)
-class wedge(MathSymbol): unicode = unichr(8743)
-class setminus(MathSymbol): unicode = unichr(8726)
-class wr(MathSymbol): unicode = unichr(8768)
-class diamond(MathSymbol): unicode = unichr(8900)
-class bigtriangleup(MathSymbol): unicode = unichr(9651)
-class bigtriangledown(MathSymbol): unicode = unichr(9661)
-class triangleleft(MathSymbol): unicode = unichr(9667)
-class triangleright(MathSymbol): unicode = unichr(9657)
+class pm(MathSymbol): str = chr(177)
+class mp(MathSymbol): str = chr(8723)
+class times(MathSymbol): str = chr(215)
+class div(MathSymbol): str = chr(247)
+class ast(MathSymbol): str = chr(42)
+class star(MathSymbol): str = chr(8902)
+class circ(MathSymbol): str = chr(9675)
+class bullet(MathSymbol): str = chr(8226)
+class cdot(MathSymbol): str = chr(183)
+class cap(MathSymbol): str = chr(8745)
+class cup(MathSymbol): str = chr(8746)
+class uplus(MathSymbol): str = chr(8846)
+class sqcap(MathSymbol): str = chr(8851)
+class sqcup(MathSymbol): str = chr(8852)
+class vee(MathSymbol): str = chr(8744)
+class wedge(MathSymbol): str = chr(8743)
+class setminus(MathSymbol): str = chr(8726)
+class wr(MathSymbol): str = chr(8768)
+class diamond(MathSymbol): str = chr(8900)
+class bigtriangleup(MathSymbol): str = chr(9651)
+class bigtriangledown(MathSymbol): str = chr(9661)
+class triangleleft(MathSymbol): str = chr(9667)
+class triangleright(MathSymbol): str = chr(9657)
 class lhd(MathSymbol): pass
 class rhd(MathSymbol): pass
 class unlhd(MathSymbol): pass
 class unrhd(MathSymbol): pass
-class oplus(MathSymbol): unicode = unichr(8853)
-class ominus(MathSymbol): unicode = unichr(8854)
-class otimes(MathSymbol): unicode = unichr(8855)
-class oslash(MathSymbol): unicode = unichr(8856)
-class odot(MathSymbol): unicode = unichr(8857)
-class bigcirc(MathSymbol): unicode = unichr(9711)
-class dagger(MathSymbol): unicode = unichr(8224)
-class ddagger(MathSymbol): unicode = unichr(8225)
-class amalg(MathSymbol): unicode = unichr(8720)
+class oplus(MathSymbol): str = chr(8853)
+class ominus(MathSymbol): str = chr(8854)
+class otimes(MathSymbol): str = chr(8855)
+class oslash(MathSymbol): str = chr(8856)
+class odot(MathSymbol): str = chr(8857)
+class bigcirc(MathSymbol): str = chr(9711)
+class dagger(MathSymbol): str = chr(8224)
+class ddagger(MathSymbol): str = chr(8225)
+class amalg(MathSymbol): str = chr(8720)
 
 #
 # Table 3.5: Relation Symbols
@@ -311,63 +323,63 @@ class amalg(MathSymbol): unicode = unichr(8720)
 class Not(MathSymbol):
     macroName = 'not'
     args = 'symbol'
-class leq(MathSymbol): unicode = unichr(8804)
-class le(MathSymbol): unicode = unichr(8804)
-class prec(MathSymbol): unicode = unichr(8826)
-class preceq(MathSymbol): unicode = unichr(8828)
-class ll(MathSymbol): unicode = unichr(8810)
-class subset(MathSymbol): unicode = unichr(8834)
-class subseteq(MathSymbol): unicode = unichr(8838)
-class sqsubseteq(MathSymbol): unicode = unichr(8849)
+class leq(MathSymbol): str = chr(8804)
+class le(MathSymbol): str = chr(8804)
+class prec(MathSymbol): str = chr(8826)
+class preceq(MathSymbol): str = chr(8828)
+class ll(MathSymbol): str = chr(8810)
+class subset(MathSymbol): str = chr(8834)
+class subseteq(MathSymbol): str = chr(8838)
+class sqsubseteq(MathSymbol): str = chr(8849)
 class In(MathSymbol):
     macroName = 'in'
-class vdash(MathSymbol): unicode = unichr(8866)
-class geq(MathSymbol): unicode = unichr(8805)
-class ge(MathSymbol): unicode = unichr(8805)
-class succ(MathSymbol): unicode = unichr(8827)
-class succeq(MathSymbol): unicode = unichr(8829)
-class gg(MathSymbol): unicode = unichr(8811)
-class supset(MathSymbol): unicode = unichr(8835)
-class supseteq(MathSymbol): unicode = unichr(8839)
-class sqsupset(MathSymbol): unicode = unichr(8848)
-class sqsupseteq(MathSymbol): unicode = unichr(8850)
-class ni(MathSymbol): unicode = unichr(8715)
-class dashv(MathSymbol): unicode = unichr(8867)
-class equiv(MathSymbol): unicode = unichr(8801)
-class sim(MathSymbol): unicode = unichr(8764)
-class simeq(MathSymbol): unicode = unichr(8771)
-class asymp(MathSymbol): unicode = unichr(8781)
-class approx(MathSymbol): unicode = unichr(8776)
-class cong(MathSymbol): unicode = unichr(8773)
-class neq(MathSymbol): unicode = unichr(8800)
-class ne(MathSymbol): unicode = unichr(8800)
-class doteq(MathSymbol): unicode = unichr(8784)
+class vdash(MathSymbol): str = chr(8866)
+class geq(MathSymbol): str = chr(8805)
+class ge(MathSymbol): str = chr(8805)
+class succ(MathSymbol): str = chr(8827)
+class succeq(MathSymbol): str = chr(8829)
+class gg(MathSymbol): str = chr(8811)
+class supset(MathSymbol): str = chr(8835)
+class supseteq(MathSymbol): str = chr(8839)
+class sqsupset(MathSymbol): str = chr(8848)
+class sqsupseteq(MathSymbol): str = chr(8850)
+class ni(MathSymbol): str = chr(8715)
+class dashv(MathSymbol): str = chr(8867)
+class equiv(MathSymbol): str = chr(8801)
+class sim(MathSymbol): str = chr(8764)
+class simeq(MathSymbol): str = chr(8771)
+class asymp(MathSymbol): str = chr(8781)
+class approx(MathSymbol): str = chr(8776)
+class cong(MathSymbol): str = chr(8773)
+class neq(MathSymbol): str = chr(8800)
+class ne(MathSymbol): str = chr(8800)
+class doteq(MathSymbol): str = chr(8784)
 class notin(MathSymbol): pass
-class models(MathSymbol): unicode = unichr(8871)
-class perp(MathSymbol): unicode = unichr(8869)
-class mid(MathSymbol): unicode = unichr(8739)
-class parallel(MathSymbol): unicode = unichr(8741)
-class bowtie(MathSymbol): unicode = unichr(8904)
+class models(MathSymbol): str = chr(8871)
+class perp(MathSymbol): str = chr(8869)
+class mid(MathSymbol): str = chr(8739)
+class parallel(MathSymbol): str = chr(8741)
+class bowtie(MathSymbol): str = chr(8904)
 class Join(MathSymbol): pass
-class smile(MathSymbol): unicode = unichr(8995)
-class frown(MathSymbol): unicode = unichr(8994)
-class propto(MathSymbol): unicode = unichr(8733)
+class smile(MathSymbol): str = chr(8995)
+class frown(MathSymbol): str = chr(8994)
+class propto(MathSymbol): str = chr(8733)
 
 #
 # Table 3.6: Arrow Symbols
 #
 
-class leftarrow(MathSymbol): unicode = unichr(8592)
-class Leftarrow(MathSymbol): unicode = unichr(8656)
-class rightarrow(MathSymbol): unicode = unichr(8594)
-class Rightarrow(MathSymbol): unicode = unichr(8658)
-class leftrightarrow(MathSymbol): unicode = unichr(8596)
-class Leftrightarrow(MathSymbol): unicode = unichr(8660)
-class mapsto(MathSymbol): unicode = unichr(8614)
-class hookleftarrow(MathSymbol): unicode = unichr(8617)
-class leftharpoonup(MathSymbol): unicode = unichr(8636)
-class leftharpoondown(MathSymbol): unicode = unichr(8637)
-class rightleftharpoons(MathSymbol): unicode = unichr(8652)
+class leftarrow(MathSymbol): str = chr(8592)
+class Leftarrow(MathSymbol): str = chr(8656)
+class rightarrow(MathSymbol): str = chr(8594)
+class Rightarrow(MathSymbol): str = chr(8658)
+class leftrightarrow(MathSymbol): str = chr(8596)
+class Leftrightarrow(MathSymbol): str = chr(8660)
+class mapsto(MathSymbol): str = chr(8614)
+class hookleftarrow(MathSymbol): str = chr(8617)
+class leftharpoonup(MathSymbol): str = chr(8636)
+class leftharpoondown(MathSymbol): str = chr(8637)
+class rightleftharpoons(MathSymbol): str = chr(8652)
 class longleftarrow(MathSymbol): pass
 class Longleftarrow(MathSymbol): pass
 class longrightarrow(MathSymbol): pass
@@ -375,68 +387,68 @@ class Longrightarrow(MathSymbol): pass
 class longleftrightarrow(MathSymbol): pass
 class Longleftrightarrow(MathSymbol): pass
 class longmapsto(MathSymbol): pass
-class hookrightarrow(MathSymbol): unicode = unichr(8618)
-class rightharpoonup(MathSymbol): unicode = unichr(8640)
-class rightharpoondown(MathSymbol): unicode = unichr(8641)
+class hookrightarrow(MathSymbol): str = chr(8618)
+class rightharpoonup(MathSymbol): str = chr(8640)
+class rightharpoondown(MathSymbol): str = chr(8641)
 class leadsto(MathSymbol): pass
-class uparrow(MathSymbol): unicode = unichr(8593)
-class Uparrow(MathSymbol): unicode = unichr(8657)
-class downarrow(MathSymbol): unicode = unichr(8595)
-class Downarrow(MathSymbol): unicode = unichr(8659)
-class updownarrow(MathSymbol): unicode = unichr(8597)
-class Updownarrow(MathSymbol): unicode = unichr(8661)
-class nearrow(MathSymbol): unicode = unichr(8599)
-class searrow(MathSymbol): unicode = unichr(8600)
-class swarrow(MathSymbol): unicode = unichr(8601)
-class nwarrow(MathSymbol): unicode = unichr(8598)
+class uparrow(MathSymbol): str = chr(8593)
+class Uparrow(MathSymbol): str = chr(8657)
+class downarrow(MathSymbol): str = chr(8595)
+class Downarrow(MathSymbol): str = chr(8659)
+class updownarrow(MathSymbol): str = chr(8597)
+class Updownarrow(MathSymbol): str = chr(8661)
+class nearrow(MathSymbol): str = chr(8599)
+class searrow(MathSymbol): str = chr(8600)
+class swarrow(MathSymbol): str = chr(8601)
+class nwarrow(MathSymbol): str = chr(8598)
 
 #
 # Table 3.7: Miscellaneous Symbols
 #
 
-class aleph(MathSymbol): unicode = unichr(8501)
-class hbar(MathSymbol): unicode = unichr(8463)
+class aleph(MathSymbol): str = chr(8501)
+class hbar(MathSymbol): str = chr(8463)
 class imath(MathSymbol): pass
 class jmath(MathSymbol): pass
-class ell(MathSymbol): unicode = unichr(8467)
-class wp(MathSymbol): unicode = unichr(8472)
-class Re(MathSymbol): unicode = unichr(8476)
-class Im(MathSymbol): unicode = unichr(8465)
-class mho(MathSymbol): unicode = unichr(8487)
-class prime(MathSymbol): unicode = unichr(8242)
-class emptyset(MathSymbol): unicode = unichr(8709)
-class nabla(MathSymbol): unicode = unichr(8711)
-class surd(MathSymbol): unicode = unichr(8730)
-class top(MathSymbol): unicode = unichr(8868)
-class bot(MathSymbol): unicode = unichr(8869)
+class ell(MathSymbol): str = chr(8467)
+class wp(MathSymbol): str = chr(8472)
+class Re(MathSymbol): str = chr(8476)
+class Im(MathSymbol): str = chr(8465)
+class mho(MathSymbol): str = chr(8487)
+class prime(MathSymbol): str = chr(8242)
+class emptyset(MathSymbol): str = chr(8709)
+class nabla(MathSymbol): str = chr(8711)
+class surd(MathSymbol): str = chr(8730)
+class top(MathSymbol): str = chr(8868)
+class bot(MathSymbol): str = chr(8869)
 class VerticalBar(MathSymbol):
     macroName = '|'
-class forall(MathSymbol): unicode = unichr(8704)
-class exists(MathSymbol): unicode = unichr(8707)
+class forall(MathSymbol): str = chr(8704)
+class exists(MathSymbol): str = chr(8707)
 class neg(MathSymbol): pass
-class flat(MathSymbol): unicode = unichr(9837)
-class natural(MathSymbol): unicode = unichr(9838)
-class sharp(MathSymbol): unicode = unichr(9839)
-class backslash(MathSymbol): unicode = unichr(92)
-class partial(MathSymbol): unicode = unichr(8706)
-class infty(MathSymbol): unicode = unichr(8734)
+class flat(MathSymbol): str = chr(9837)
+class natural(MathSymbol): str = chr(9838)
+class sharp(MathSymbol): str = chr(9839)
+class backslash(MathSymbol): str = chr(92)
+class partial(MathSymbol): str = chr(8706)
+class infty(MathSymbol): str = chr(8734)
 class Box(MathSymbol): pass
 class Diamond(MathSymbol): pass
-class triangle(MathSymbol): unicode = unichr(9653)
-class clubsuit(MathSymbol): unicode = unichr(9827)
-class diamondsuit(MathSymbol): unicode = unichr(9830)
-class heartsuit(MathSymbol): unicode = unichr(9829)
-class spadesuit(MathSymbol): unicode = unichr(9824)
+class triangle(MathSymbol): str = chr(9653)
+class clubsuit(MathSymbol): str = chr(9827)
+class diamondsuit(MathSymbol): str = chr(9830)
+class heartsuit(MathSymbol): str = chr(9829)
+class spadesuit(MathSymbol): str = chr(9824)
 
 #
 # Table 3.8: Variable-sized Symbols
 #
 
-class sum(MathSymbol): unicode = unichr(8721)
-class prod(MathSymbol): unicode = unichr(8719)
-class coprod(MathSymbol): unicode = unichr(8720)
-class int(MathSymbol): unicode = unichr(8747)
-class oint(MathSymbol): unicode = unichr(8750)
+class sum(MathSymbol): str = chr(8721)
+class prod(MathSymbol): str = chr(8719)
+class coprod(MathSymbol): str = chr(8720)
+class int(MathSymbol): str = chr(8747)
+class oint(MathSymbol): str = chr(8750)
 class bigcap(MathSymbol): pass
 class bigcup(MathSymbol): pass
 class bigsqcup(MathSymbol): pass
@@ -562,8 +574,8 @@ class brack(Command):
 class brace(Command):
     pass
 
-class sqrt(Command):
-    pass
+#class sqrt(Command):
+#    pass
 
 #
 # C.7.6 Putting One Thing Above Another
