@@ -5,9 +5,8 @@ C.3.2 Making Paragraphs (p171)
 
 """
 
-from plasTeX.Tokenizer import Other
-from plasTeX import Command, Environment, DimenCommand
-from plasTeX.Logging import getLogger
+from plasTeX import Command, DimenCommand
+
 
 class noindent(Command):
     pass
@@ -39,7 +38,7 @@ class baselineskip(DimenCommand):
     value = DimenCommand.new('12pt')
 
 class baselinestretch(Command):
-    unicode = '1'
+    str = '1'
 
 class parskip(DimenCommand):
     value = DimenCommand.new(0)
