@@ -5,9 +5,8 @@ C.3.1 Making Sentences (p170)
 
 """
 
-import time
 from plasTeX import Command, Environment
-from plasTeX.Logging import getLogger
+
 
 #
 # Quotes
@@ -29,19 +28,19 @@ from plasTeX.Logging import getLogger
 
 class SmallSpace(Command):
     macroName = ','
-    unicode = u'\u2009'
+    str = '\u2009'
 
 class InterWordSpace(Command):
     macroName = ' '
-    unicode = u'\u0020'
+    str = '\u0020'
 
 class NoLineBreak(Command):
     macroName = 'active::~'
-    unicode = u'\u00A0'
+    str = '\u00A0'
 
 class EndOfSentence(Command):
     macroName = '@'
-    unicode = u''
+    str = ''
 
 class frenchspacing(Command): 
     pass
@@ -55,31 +54,31 @@ class nonfrenchspacing(Command):
 
 class Dollar(Command):
     macroName = '$'
-    unicode = '$'
+    str = '$'
 
 class Percent(Command):
     macroName = '%'
-    unicode = '%'
+    str = '%'
 
 class LeftBrace(Command):
     macroName = '{'
-    unicode = '{'
+    str = '{'
 
 class Underscore(Command):
     macroName = '_'
-    unicode = '_'
+    str = '_'
 
 class Ampersand(Command):
     macroName = '&'
-    unicode = '&'
+    str = '&'
 
 class HashMark(Command):
     macroName = '#'
-    unicode = '#'
+    str = '#'
 
 class RightBrace(Command):
     macroName = '}'
-    unicode = '}'
+    str = '}'
 
 #
 # Logos
@@ -95,12 +94,15 @@ class TeX(Command):
 # Misc
 #
 
-#class today(Command):
-#    unicode = time.strftime('%B %d, %Y')
-
 class emph(Command):
     args = 'self'
 
 class em(Environment):
     pass
+    
+class textsubscript(Command):
+     args = 'self'
+ 
+class textsuperscript(Command):
+    args = 'self'
 
