@@ -1,9 +1,11 @@
 #!/usr/bin/env python
 
 import re, os, copy
-from plasTeX.ConfigManager import BUILTIN, CODE, ENVIRON, ENVIRONMENT
-from plasTeX.ConfigManager import CODE, REGISTRY, COMMANDLINE, InvalidOptionError
+from plasTeX.ConfigManager import BUILTIN, CODE, ENVIRONMENT
+from plasTeX.ConfigManager import COMMANDLINE, InvalidOptionError
 from plasTeX.ConfigManager import ConfigManager, TooManyValues, TooFewValues
+import collections.abc
+
 
 DEFAULTS = \
 {
@@ -34,7 +36,7 @@ class GenericParser:
       """ Return boolean indicating the existence of another argument """
       if not ConfigManager.has_following_argument(args):
          return 0
-      
+
       # No arguments left
       if not args:
          return 0
@@ -49,10 +51,10 @@ class GenericParser:
 
       return 0
 
-   def getArgument(self, args, range=[1,1], delim=','):
+   def getArgument(self, args, range=None, delim=',', forcedarg=False):
       """ Parse argument """
 
-      range = self.validateRange(range[:])
+      range = self.validateRange(range or [1, 1])
 
       # If we have an optional default, make sure that the range reflects that
       if range[0] < 2 and self.optional not in [None, [], ()]:
@@ -68,11 +70,11 @@ class GenericParser:
             name = self.name
             if self.actual: name = self.actual
             if range[0] != range[1]:
-               raise TooFewValues, "Expecting at least %s value(s) for '%s'." % (range[0], name)
+               raise TooFewValues("Expecting at least %s value(s) for '%s'." % (range[0], name))
             elif range[0] == 1:
-               raise TooFewValues, "Expecting a value for '%s'." % name
+               raise TooFewValues("Expecting a value for '%s'." % name)
             else:
-               raise TooFewValues, "Expecting %s values for '%s'." % (range[0], name)
+               raise TooFewValues("Expecting %s values for '%s'." % (range[0], name))
          if self.optional:
             return self.optional, args
          else:
@@ -86,10 +88,12 @@ class GenericParser:
       else:
          new_args = []
          while self._hasFollowingArgument(args, delim):
+            forcedarg = False
+
             # If the current argument ends with a delimiter
             if args[0].strip()[-1] == delim:
                new_args.append(args.pop(0).strip()[:-1])
-   
+
             # If next argument is an isolated delimiter character
             elif len(args) > 1 and args[1].strip() == delim:
                new_args.append(args.pop(0).strip())
@@ -101,16 +105,16 @@ class GenericParser:
                    new_args.append(args.pop(0).strip()[1:])
                else:
                    new_args.append(args.pop(0).strip())
-   
+
                # If this argument ends with a delimiter, rip it off
                # and put it back into the stream.
                if new_args[-1].endswith(delim):
                   args.insert(0, delim)
                   new_args[-1] = new_args[-1][:-1]
-                   
+
          # The following argument is always part of the list
          # unless the user accidentally put a trailing delimiter.
-         if ConfigManager.has_following_argument(args):
+         if forcedarg or ConfigManager.has_following_argument(args):
             new_args.append(args.pop(0).strip())
 
          if new_args and new_args[-1].startswith(delim):
@@ -131,12 +135,12 @@ class GenericParser:
          plural = 's'
          if range[0] == 1:
             plural = ''
-         raise TooFewValues, "Expecting at least %s value%s for '%s'." % (range[0], plural, name)
+         raise TooFewValues("Expecting at least %s value%s for '%s'." % (range[0], plural, name))
       elif len(new_args) > range[1]:
-         raise TooManyValues, "Expecting at most %s values for '%s'." % (range[1], name)
+         raise TooManyValues("Expecting at most %s values for '%s'." % (range[1], name))
 
       # Collapse argument list to a value if possible
-      if len(new_args) < 1:
+      if not new_args:
          if self.optional is not None:
             new_args = self.optional
          else:
@@ -144,11 +148,11 @@ class GenericParser:
       elif len(new_args) < 2:
          new_args = new_args[0]
 
-      return new_args, args 
+      return new_args, args
 
-   def _getSpaceDelimitedArguments(self, args, range=[1,'*']):
+   def _getSpaceDelimitedArguments(self, args, range=None):
       """ Slurp up any following arguments that don't begin with '-' """
-      minimum, maximum = self.validateRange(range)
+      minimum, maximum = self.validateRange(range or [1, '*'])
       arg_len = len(args)
       new_args = []
       while ConfigManager.has_following_argument(args):
@@ -161,7 +165,7 @@ class GenericParser:
           return new_args, args
       else:
           if len(new_args) == arg_len:
-              minimum = min(len(new_args), minimum) 
+              minimum = min(len(new_args), minimum)
               return new_args[:minimum], new_args[minimum:]
           else:
               return new_args, args
@@ -176,7 +180,7 @@ class GenericParser:
       range[0] = int(range[0])
       range[1] = int(range[1])
       return range
- 
+
 
 class GenericOption(object):
    """
@@ -203,7 +207,7 @@ class GenericOption(object):
                       mandatory=None,
                       name=DEFAULTS['name'],
                       source=DEFAULTS['source']):
-      """
+      r"""
       Declare a command line option
 
       Instances of subclasses of CommandLineOption must be placed in
@@ -238,10 +242,10 @@ class GenericOption(object):
 
       values -- valid values for the option.  This argument can take
          the following forms:
-        
+
          single value -- for StringOption this this is a string, for
             IntegerOption this is an integer, for FloatOption this is
-            a float.  The single value mode is most useful when the 
+            a float.  The single value mode is most useful when the
             value is a regular expression.  For example, to specify
             that a StringOption must be a string of characters followed
             by a digit, 'values' would be set to re.compile(r'\w+\d').
@@ -250,17 +254,17 @@ class GenericOption(object):
             the endpoints of a range of valid values.  This is probably
             most useful on IntegerOption and FloatOption.  For example,
             to specify that an IntegerOption can only take the values
-            from 0 to 10, 'values' would be set to [0,10]. 
+            from 0 to 10, 'values' would be set to [0,10].
 
             NOTE: This mode must *always* use a Python list since using
                   a tuple means something else entirely.
 
-         tuple of values -- a tuple of values can be used to specify 
+         tuple of values -- a tuple of values can be used to specify
             a complete list of valid values.  For example, to specify
             that an IntegerOption can take the values 1, 2, or 3, 'values'
-            would be set to (1,2,3).  If a string value can only take 
+            would be set to (1,2,3).  If a string value can only take
             the values, 'hi', 'bye', and any string of characters beginning
-            with the letter 'z', 'values' would be set to 
+            with the letter 'z', 'values' would be set to
             ('hi','bye',re.compile(r'z.*?')).
 
             NOTE: This mode must *always* use a Python tuple since using
@@ -268,7 +272,7 @@ class GenericOption(object):
 
       category -- a category key which specifies which category the
          option belongs to (see Option.add_category())
-      
+
       callback -- a function to call after the value of the option has
          been validated.  This function will be called with the validated
          option value as its only argument.
@@ -370,8 +374,8 @@ class GenericOption(object):
 
       """
       if type(self.__class__) is GenericOption:
-         raise ValueError, 'GenericOption cannot be instantiated ' + \
-                           'directly, you must use a subclass.'
+         raise ValueError('GenericOption cannot be instantiated ' + \
+                           'directly, you must use a subclass.')
       self.parent = None
       self.initialize(locals())
       self.data = None
@@ -390,10 +394,11 @@ class GenericOption(object):
       synopsis = vars.get('synopsis', DEFAULTS['synopsis'])
       environ = vars.get('environ', DEFAULTS['environ'])
       mandatory = vars.get('mandatory')
+
       if mandatory is None:
          mandatory = isinstance(self, GenericArgument)
       registry = vars.get('registry', DEFAULTS['registry'])
-      
+
       self.actual = None  # Actual option used on the command-line
       self.name = name
       self.options = options
@@ -422,7 +427,7 @@ class GenericOption(object):
                        # set if self.source == CONFIGFILE.
 
       if source <= ENVIRONMENT:
-         if self.environ and os.environ.has_key(str(self.environ)):
+         if self.environ and str(self.environ) in os.environ:
             self.default = self.cast(os.environ[str(self.environ)])
             self.source = ENVIRONMENT
          else:
@@ -442,7 +447,7 @@ class GenericOption(object):
          return self.data
 
       # See if it was set in the environment
-      if self.environ and os.environ.has_key(str(self.environ)):
+      if self.environ and str(self.environ) in os.environ:
          self.source = ENVIRONMENT
          self.file = None
          return self.cast(os.environ[str(self.environ)])
@@ -474,7 +479,7 @@ class GenericOption(object):
    def copy(self):
       """ Make a deep copy of self """
       newcopy = self.__class__()
-      for key, value in vars(self).copy().items():
+      for key, value in list(vars(self).copy().items()):
          if key == 'data':
             setattr(newcopy, key, copy.copy(value))
          else:
@@ -556,24 +561,21 @@ class GenericOption(object):
 
       """
       return {}
-      return {'name': self.name, 'default': self.default, 
-              'option': self.actual, 'synopsis': self.synopsis}
+      #return {'name': self.name, 'default': self.default,
+              #'option': self.actual, 'synopsis': self.synopsis}
 
    def __lt__(self, other): return self.compare(other) < 0
    def __gt__(self, other): return self.compare(other) > 0
    def __le__(self, other): return self.compare(other) <= 0
    def __ge__(self, other): return self.compare(other) >= 0
    def __eq__(self, other): return self.compare(other) == 0
-   def __cmp__(self, other): return self.compare(other)
+   
 
    def compare(self, other):
       """ Compare option to another using the specified sort order """
       for attr in self.sort_order:
-          sattr = getattr(self, attr)
-          oattr = getattr(other, attr)
-
-          if sattr == None: sattr = ''
-          if oattr == None: oattr = ''
+          sattr = getattr(self, attr, '')
+          oattr = getattr(other, attr, '')
 
           if type(sattr) is str and type(oattr) is str:
               sattr = re.sub(r'\W', '', sattr)
@@ -601,7 +603,7 @@ class GenericOption(object):
       if value is None:
          self.clearValue()
       else:
-         if callable(self.callback):
+         if isinstance(self.callback, collections.abc.Callable):
             value = self.callback(self.cast(value))
          self.data = self.validate(value)
 
@@ -668,7 +670,7 @@ class GenericOption(object):
             return value
 
          if self.error:
-            raise InvalidOptionError(name, value, 
+            raise InvalidOptionError(name, value,
                                      msg=self.error % self.names())
          else:
             raise InvalidOptionError(name, value,
@@ -687,11 +689,11 @@ class GenericOption(object):
 
          # Look for regular expressions in the list
          for regex in [x for x in self.values if isinstance(x, RegexType)]:
-            if regex.search(value): 
+            if regex.search(value):
                return value
-            
+
          if self.error:
-            raise InvalidOptionError(name, value, 
+            raise InvalidOptionError(name, value,
                                      msg=self.error % self.names())
          else:
             raise InvalidOptionError(name, value,
@@ -699,13 +701,12 @@ class GenericOption(object):
                       (', '.join(map(str, self.values))))
 
       # Check to see if the value is equal to the only valid value
-      elif isinstance(self.values, str) or isinstance(self.values, int) \
-           or isinstance(self.values, float):
+      elif isinstance(self.values, (str, int, float)):
          if value == self.values:
             return value
 
          if self.error:
-            raise InvalidOptionError(name, value, 
+            raise InvalidOptionError(name, value,
                                      msg=self.error % self.names())
          else:
             raise InvalidOptionError(name, value,
@@ -720,26 +721,26 @@ class GenericOption(object):
             return value
 
          if self.error:
-            raise InvalidOptionError(name, value, 
+            raise InvalidOptionError(name, value,
                                      msg=self.error % self.names())
          else:
             raise InvalidOptionError(name, value,
                                      msg='Given value is not a valid value')
 
       # Use the user defined validation function
-      elif callable(self.values):
+      elif isinstance(self.values, collections.abc.Callable):
          if self.values(value):
             return value
 
          if self.error:
-            raise InvalidOptionError(name, value, 
+            raise InvalidOptionError(name, value,
                                      msg=self.error % self.names())
          else:
             raise InvalidOptionError(name, value,
                                      msg='Given value is not a valid value')
 
       # Bail out if we don't know what this is
-      raise ValueError, 'Unknown valid values type'
+      raise ValueError('Unknown valid values type')
 
 
 class GenericArgument(GenericOption):
