@@ -1,10 +1,11 @@
 #!/usr/bin/env python
 
 import os
-from Generic import *
+from plasTeX.ConfigManager.Generic import *
 from plasTeX.ConfigManager import TooManyValues
-from String import StringOption
-from UserList import UserList
+from plasTeX.ConfigManager.String import StringOption
+from collections import UserList
+import collections.abc
 
 
 class MultiParser(GenericParser):
@@ -38,7 +39,7 @@ class MultiOption(MultiParser, GenericOption, UserList):
 
    def __init__(self, docstring=DEFAULTS['docstring'],
                       options=DEFAULTS['options'],
-                      default=[],
+                      default=None,
                       optional=DEFAULTS['optional'],
                       values=DEFAULTS['values'],
                       category=DEFAULTS['category'],
@@ -47,7 +48,7 @@ class MultiOption(MultiParser, GenericOption, UserList):
                       environ=DEFAULTS['environ'],
                       registry=DEFAULTS['registry'],
                       delim=',',
-                      range=[1,'*'],
+                      range=None,
                       mandatory=None,
                       name=DEFAULTS['name'],
                       source=DEFAULTS['source'],
@@ -63,7 +64,8 @@ class MultiOption(MultiParser, GenericOption, UserList):
 
       """
       self.delim = delim
-      self.range = range
+      default = default = []
+      self.range = range or [1, '*']
       assert not issubclass(template, MultiOption), \
              'MultiOptions can not have a MultiOption as a template'
       assert issubclass(template, GenericOption), \
@@ -87,7 +89,7 @@ class MultiOption(MultiParser, GenericOption, UserList):
       if self.data and self.source & COMMANDLINE:
          return self.data
 
-      if self.environ and os.environ.has_key(str(self.environ)):
+      if self.environ and str(self.environ) in os.environ:
          self.source = ENVIRONMENT
          self.file = None
          return self.cast(os.environ[str(self.environ)])
@@ -128,16 +130,16 @@ class MultiOption(MultiParser, GenericOption, UserList):
 
    def __iadd__(self, other):
       """ Append a value to the list """
-      if callable(self.callback):
+      if isinstance(self.callback, collections.abc.Callable):
          other = self.callback(self.cast(other))
       self.data += self.validate(other)
       range = self.validateRange(self.range)
       name = self.name
       if self.actual: name = self.actual
       if len(self.data) > range[1]:
-         raise TooManyValues, "Expecting at most %s values for option '%s'." % (range[1], name)
+         raise TooManyValues("Expecting at most %s values for option '%s'." % (range[1], name))
       return self
- 
+
    def validate(self, arg):
       """ Validate the value of the option """
       new_values = []
@@ -148,7 +150,7 @@ class MultiOption(MultiParser, GenericOption, UserList):
 
    def checkValues(self, value):
       return self.template.checkValues(value)
-  
+
    def __str__(self):
       if self.delim and self.delim.strip():
          delim = '%s ' % self.delim
@@ -160,12 +162,12 @@ class MultiOption(MultiParser, GenericOption, UserList):
    def acceptsArgument(self):
       """ Return a boolean indicating if the option accepts an argument """
       range = self.validateRange(self.range)
-      return not(not(range[1]))
+      return bool(range[1])
 
    def requiresArgument(self):
       """ Return a boolean indicating if the option requires an argument """
       range = self.validateRange(self.range)
-      return not(not(range[0]))
+      return bool(range[0])
 
    def setValue(self, value):
       """
@@ -179,7 +181,7 @@ class MultiOption(MultiParser, GenericOption, UserList):
       if value is None or ((type(value) in [list,tuple]) and not(value)):
          self.clearValue()
       else:
-         if callable(self.callback):
+         if isinstance(self.callback, collections.abc.Callable):
             value = self.callback(self.cast(value))
          self.data = self.validate(value)
 
@@ -189,7 +191,7 @@ class MultiArgument(GenericArgument, MultiOption):
 
    def __init__(self, docstring=DEFAULTS['docstring'],
                       options=DEFAULTS['options'],
-                      default=[],
+                      default=None,
                       optional=DEFAULTS['optional'],
                       values=DEFAULTS['values'],
                       category=DEFAULTS['category'],
@@ -198,14 +200,15 @@ class MultiArgument(GenericArgument, MultiOption):
                       environ=DEFAULTS['environ'],
                       registry=DEFAULTS['registry'],
                       delim=' ',
-                      range=[1,'*'],
+                      range=None,
                       mandatory=None,
                       name=DEFAULTS['name'],
                       source=DEFAULTS['source'],
                       template=StringOption):
       """ Initialize a multi argument """
       self.delim = delim
-      self.range = range
+      default = default or []
+      self.range = range or [1, '*']
       assert not issubclass(template, MultiArgument), \
              'MultiArguments can not have a MultiArguments as a template'
       assert not issubclass(template, MultiOption), \
