@@ -1,11 +1,13 @@
 #!/usr/bin/env python
 
-import new, os, ConfigParser, re, time, codecs
-import plasTeX
 from plasTeX import ismacro, macroName
-from plasTeX.DOM import Node
 from plasTeX.Logging import getLogger
-from Tokenizer import Tokenizer, Token, DEFAULT_CATEGORIES, VERBATIM_CATEGORIES
+from plasTeX.Tokenizer import Tokenizer, Token, DEFAULT_CATEGORIES, VERBATIM_CATEGORIES
+import os
+import configparser
+import re
+import time
+import plasTeX
 
 # Only export the Context singleton
 __all__ = ['Context']
@@ -18,13 +20,13 @@ macrolog = getLogger('context.macros')
 
 
 class ContextItem(dict):
-    """ 
+    """
     Localized macro/category code stack element
 
     """
 
-    def __init__(self, data={}):
-        dict.__init__(self, data)
+    def __init__(self, data=None):
+        dict.__init__(self, data or {})
         self.categories = None
         self.obj = None
         self.parent = None
@@ -37,7 +39,7 @@ class ContextItem(dict):
         return '{}'
 
     def __getitem__(self, key):
-        try: 
+        try:
             return dict.__getitem__(self, key)
         except KeyError:
             if self.parent is not None and self.parent is not self:
@@ -49,21 +51,21 @@ class ContextItem(dict):
         except KeyError: return default
 
     def has_key(self, key):
-        if dict.has_key(self, key):
+        if dict.__contains__(self, key):
             return True
         if self.parent is not None:
-            return self.parent.has_key(key)
+            return key in list(self.parent.keys())
 
     __contains__ = has_key
 
     def keys(self):
         keys = {}
-        for key in dict.keys(self):
+        for key in list(dict.keys(self)):
             keys[key] = 0
         if self.parent is not None:
-            for key in self.parent.keys():
+            for key in list(self.parent.keys()):
                 keys[key] = 0
-        return keys.keys()
+        return list(keys.keys())
 
     def __str__(self):
         if self.parent is not None:
@@ -73,7 +75,7 @@ class ContextItem(dict):
 
 class Counters(dict):
     def __getitem__(self, name):
-        try: 
+        try:
             c = dict.__getitem__(self, name)
         except KeyError:
 #           log.warning('No counter "%s" exists.  Creating one.' % name)
@@ -84,35 +86,37 @@ class Counters(dict):
 class LanguageParser(object):
     """ Parser for language commands """
 
-    def __init__(self, output={}):
-        self.data = output
+    def __init__(self, output=None):
+        self.data = output if output is not None else {}
         self.language = None
         self.term = None
 
     def parse(self, files, encoding='UTF-8'):
         from xml.parsers import expat
-        if isinstance(files, basestring):
+        if isinstance(files, str):
             files = [files]
-        for file in files:
-            if not os.path.isfile(file):
+        for fname in files:
+            if not os.path.isfile(fname):
                 continue
-            self.parser = expat.ParserCreate('UTF-8')
+
+            self.parser = expat.ParserCreate(encoding)
             self.parser.StartElementHandler = self.startElement
             self.parser.EndElementHandler = self.endElement
             self.parser.CharacterDataHandler = self.charData
-            self.parser.Parse(codecs.open(file, 'r', encoding).read().encode('UTF-8'))
+            with open(fname, encoding=encoding) as f:
+                self.parser.Parse(f.read())
         self.mergeLanguages()
         return self.data
 
     def mergeLanguages(self):
         # Merge language keys from the major language section, into
         # the minor language section
-        for key, value in self.data.items():
+        for key, value in list(self.data.items()):
             if '-' in key:
-                major, minor = key.split('-',1)
-                if major in self.data:
+                major, minor = key.split('-', 1)
+                if major in list(self.data.keys()):
                     majordict = self.data[major]
-                    for mkey, mvalue in majordict.items():
+                    for mkey, mvalue in list(majordict.items()):
                         if mkey not in value:
                             value[mkey] = mvalue
 
@@ -120,19 +124,19 @@ class LanguageParser(object):
         if name == 'terms':
             self.term = None
             if self.data.get(attrs['lang']):
-                self.language = self.data[attrs['lang']] 
+                self.language = self.data[attrs['lang']]
             else:
                 self.language = self.data[attrs['lang']] = {}
             if 'babel' in attrs:
                 self.data[attrs['babel']] = self.language
         elif name == 'term':
             self.term = attrs['name']
-            self.language[self.term] = u''
-            
+            self.language[self.term] = ''
+
     def endElement(self, name):
         if name == 'term':
             self.term = None
-            
+
     def charData(self, data):
         if self.term:
             self.language[self.term] += data
@@ -145,7 +149,7 @@ class Context(object):
     This class keeps track of macros (both global and local), labels,
     context groupings, category codes, etc.  The TeX parser uses this
     class to hold any and all information about the document currently
-    being processed.  This class also contains methods to generate 
+    being processed.  This class also contains methods to generate
     new counters, ifs, dimensions, and other commands and macros.
 
     """
@@ -220,7 +224,7 @@ class Context(object):
 
         Required Arguments:
         filename -- the name of the file with the shelved data
- 
+
         Keyword Arguments:
         rtype -- the key in the shelved data to look under.  This is generally
             the name of the renderer used since the information for each
@@ -230,8 +234,8 @@ class Context(object):
         import pickle
         if os.path.exists(filename):
             try:
-                d = pickle.load(open(filename,'rb'))
-                if rtype not in d:
+                d = pickle.load(open(filename, 'rb'))
+                if rtype not in list(d.keys()):
                     d[rtype] = {}
             except:
                 os.remove(filename)
@@ -239,11 +243,11 @@ class Context(object):
         else:
             d = {rtype:{}}
         data = d[rtype]
-        for key, value in self.persistentLabels.items():
+        for key, value in list(self.persistentLabels.items()):
             data[key] = value.persist()
         try:
-            pickle.dump(d, open(filename,'wb'))
-        except Exception, msg:
+            pickle.dump(d, open(filename, 'wb'))
+        except Exception as msg:
             log.warning('Could not save auxiliary information. (%s)' % msg)
 
     def restore(self, filename, rtype='none'):
@@ -252,7 +256,7 @@ class Context(object):
 
         Required Arguments:
         filename -- the name of the file with the shelved data
- 
+
         Keyword Arguments:
         rtype -- the key in the shelved data to look under.  This is generally
             the name of the renderer used since the information for each
@@ -263,23 +267,23 @@ class Context(object):
         if not os.path.exists(filename):
             return
         try:
-            d = pickle.load(open(filename,'rb'))
+            d = pickle.load(open(filename, 'rb'))
             try: data = d[rtype]
             except KeyError: return
             wou = self.warnOnUnrecognized
             self.warnOnUnrecognized = False
-            for key, value in data.items():
-                n = self[value.get('macroName','Macro')]()
+            for key, value in list(data.items()):
+                n = self[value.get('macroName', 'Macro')]()
                 n.restore(value)
                 self.labels[key] = n
             self.warnOnUnrecognized = wou
-        except Exception, msg:
+        except Exception as msg:
             log.warning('Could not load auxiliary information. (%s)' % msg)
 
     @property
     def isMathMode(self):
         """ Are we in math mode or not? """
-        for i in range(len(self.contexts)-1, -1, -1):
+        for i in range(len(self.contexts) - 1, -1, -1):
             obj = self.contexts[i].obj
             if obj is not None and obj.mathMode is not None:
                 return obj.mathMode
@@ -298,18 +302,20 @@ class Context(object):
         lang -- the name of the language file to load
 
         """
+
         if not self.languages:
             files = document.config['document']['lang-terms'].split(os.pathsep)
             files.append(os.path.join(os.path.dirname(__file__), 'i18n.xml'))
+
             LanguageParser(self.languages).parse(reversed(files))
 
-        if lang in self.languages:
+        if lang in list(self.languages.keys()):
             self.currentLanguage = lang
             self.newcommand('languagename', definition=lang)
             self.terms = self.languages[lang]
-            for key, value in self.languages[lang].items():
+            for key, value in list(self.languages[lang].items()):
                 if key == 'today':
-                    self.newcommand(key, definition=self._strftime(value))                
+                    self.newcommand(key, definition=self._strftime(value))
                 else:
                     self.newcommand('%sname' % key, definition=value)
         else:
@@ -326,19 +332,19 @@ class Context(object):
             elif day.endswith('3'):
                 suffix = 'rd'
             day = str(int(day))
-            return unicode(time.strftime(fmt.replace('%f', day+suffix).replace('%e', day)), 'utf8')
-        return unicode(time.strftime(fmt), 'utf8')
+            return time.strftime(fmt.replace('%f', day + suffix).replace('%e', day))
+        return time.strftime(fmt)
 
     def loadINIPackage(self, inifile):
-        """ 
+        """
         Load INI file containing macro definitions
 
         Arguments:
         inifile -- filename of INI formatted file
 
         """
-        ini = ConfigParser.RawConfigParser()
-        if not isinstance(inifile, (list,tuple)):
+        ini = configparser.RawConfigParser()
+        if not isinstance(inifile, (list, tuple)):
             inifile = [inifile]
         for f in inifile:
             ini.read(f)
@@ -349,22 +355,22 @@ class Context(object):
                     log.warning('Could not find macro %s' % section)
                     continue
                 for name in ini.options(section):
-                    value = ini.get(section,name)
-                    m = re.match(r'^unicode\(\s*(?:(\'|\")(?P<string>.+)(?:\1)|(?P<number>\d+))\s*\)$',value)
+                    value = ini.get(section, name)
+                    m = re.match(r'^str\(\s*(?:(\'|\")(?P<string>.+)(?:\1)|(?P<number>\d+))\s*\)$', value)
                     if m:
                         data = m.groupdict()
                         if data['number'] is not None:
-                            value = unichr(int(data['number']))
+                            value = chr(int(data['number']))
                         else:
-                            value = unicode(data['string'])
-                        macros[name] = new.classobj(name, (baseclass,), 
-                                                    {'unicode': value})
-                        continue 
-                    macros[name] = new.classobj(name, (baseclass,), 
+                            value = data['string']
+                        macros[name] = type(name, (baseclass,),
+                                                    {'str': value})
+                        continue
+                    macros[name] = type(name, (baseclass,),
                                                 {'args': value})
             self.importMacros(macros)
 
-    def loadPackage(self, tex, file, options={}):
+    def loadPackage(self, tex, file, options=None):
         """
         Load a Python or LaTeX package
 
@@ -384,34 +390,35 @@ class Context(object):
         boolean indicating whether or not the package loaded successfully
 
         """
+        options = options or {}
         module = os.path.splitext(file)[0]
-
         # See if it has already been loaded
-        if self.packages.has_key(module):
+        if module in list(self.packages.keys()):
             return True
 
-        packagesini = os.path.join(os.path.dirname(plasTeX.Packages.__file__), 
-                                   os.path.basename(module)+'.ini')
+        packagesini = os.path.join(os.path.dirname(plasTeX.Packages.__file__),
+                                   os.path.basename(module) + '.ini')
 
-        try: 
+        try:
             # Try to import a Python package by that name
             m = __import__(module, globals(), locals())
             status.info(' ( %s ' % m.__file__)
             if hasattr(m, 'ProcessOptions'):
-                m.ProcessOptions(options or {}, tex.ownerDocument)
+                m.ProcessOptions(options, tex.ownerDocument)
+
             self.importMacros(vars(m))
-            moduleini = os.path.splitext(m.__file__)[0]+'.ini'
+            moduleini = os.path.splitext(m.__file__)[0] + '.ini'
             self.loadINIPackage([packagesini, moduleini])
             self.packages[module] = options
             status.info(' ) ')
             return True
 
-        except ImportError, msg:
+        except ImportError as msg:
+            log.warning('No Python version of %s was found' % file)
             # No Python module
-            if 'No module' in str(msg):
+            if 'No module' in str(msg) and module in str(msg):
                 pass
                 # Failed to load Python package
-#               log.warning('No Python version of %s was found' % file)
             # Error while importing
             else:
                 raise
@@ -419,14 +426,14 @@ class Context(object):
         result = tex.loadPackage(file, options)
         try:
             moduleini = os.path.join(os.path.dirname(tex.kpsewhich(file)),
-                                     os.path.basename(module)+'.ini')
+                                     os.path.basename(module) + '.ini')
             self.loadINIPackage([packagesini, moduleini])
         except OSError: pass
         return result
-   
+
 
     def label(self, label, node=None):
-        """ 
+        """
         Set a label to the current labelable object
 
         Required Arguments:
@@ -451,12 +458,12 @@ class Context(object):
             self.persistentLabels[label] = self.labels[label] = node
             node.id = label
 
-        #print label, ''.join(self.currentlabel.ref[:])
+        # print label, ''.join(self.currentlabel.ref[:])
 
         # Resolve any outstanding references to this object
-        if self.refs.has_key(label) and self.labels.has_key(label):
+        if label in list(self.refs.keys()) and label in (self.labels.keys()):
             for obj in self.refs[label]:
-                for key, value in obj.idref.items():
+                for key, value in list(obj.idref.items()):
                     if value.id != label:
                         continue
                     obj.idref[key] = self.labels[label]
@@ -480,12 +487,12 @@ class Context(object):
             return
 
         # Resolve ref if label already exists
-        if self.labels.has_key(label):
+        if label in list(self.labels.keys()):
             obj.idref[name] = self.labels[label]
-            return 
+            return
 
         # If the label doesn't exist, store away the object for later
-        if not self.refs.has_key(label):
+        if label not in list(self.refs.keys()):
             self.refs[label] = []
         self.refs[label].append(obj)
 
@@ -495,8 +502,8 @@ class Context(object):
         obj.idref[name] = node
 
     def __getitem__(self, key):
-        """ 
-        Look through the stack of macros and return the requested one 
+        """
+        Look through the stack of macros and return the requested one
 
         Required Arguments:
         key -- name of macro
@@ -510,12 +517,12 @@ class Context(object):
         # Didn't find it, so generate a new class
         if self.warnOnUnrecognized and not self.isMathMode:
             log.warning('unrecognized command/environment: %s', key)
-        self[key] = newclass = new.classobj(str(key), (plasTeX.UnrecognizedMacro,), {})
+        self[key] = newclass = type(key, (plasTeX.UnrecognizedMacro,), {})
         return newclass
 
     def push(self, context=None):
-        """ 
-        Add a new context to the stack 
+        """
+        Add a new context to the stack
 
         This adds a new context grouping to the stack.  A context
         grouping includes both a set of localized macros and localized
@@ -536,7 +543,7 @@ class Context(object):
         else:
             name = '{}'
             if context is not None:
-                name = context.nodeName 
+                name = context.nodeName
                 # If we hit a document element, make sure that we start
                 # at the global context.
                 if context.level == context.DOCUMENT_LEVEL:
@@ -549,6 +556,10 @@ class Context(object):
 
     append = push
 
+    def __contains__(self, key):
+        return key in self.top
+
+
     def mapMethods(self):
         # Getter methods use the most local context
         self.top = top = self.contexts[-1]
@@ -580,7 +591,7 @@ class Context(object):
             instance are used.  If this argument isn't supplied,
             an empty context is created.
 
-        Returns: ContextItem instance 
+        Returns: ContextItem instance
 
         """
         newcontext = ContextItem()
@@ -598,22 +609,22 @@ class Context(object):
         return newcontext
 
     def importMacros(self, context):
-        """ 
+        """
         Import macros from given context into the global namespace
 
         Required Arguments:
         context -- dictionary of macros to import
 
         """
-        for value in context.values():
+        for value in list(context.values()):
             if ismacro(value):
                 self[macroName(value)] = value
 #           elif isinstance(value, Context):
 #               self.importMacros(value)
 
     def pop(self, obj=None):
-        """ 
-        Remove a context from the stack 
+        """
+        Remove a context from the stack
 
         Keyword Arguments:
         index -- index of context item to remove
@@ -655,8 +666,8 @@ class Context(object):
         self.mapMethods()
 
     def addGlobal(self, key, value):
-        """ 
-        Add a macro to the global context 
+        """
+        Add a macro to the global context
 
         Required Arguments:
         key -- name of macro to add
@@ -666,22 +677,21 @@ class Context(object):
             instance before being added.
 
         """
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             newvalue = plasTeX.Command()
-            newvalue.unicode = value
+            newvalue.str = value
             value = newvalue
 
         elif not ismacro(value):
-            raise ValueError, \
-                  '"%s" does not implement the macro interface' % key
+            raise ValueError('"%s" does not implement the macro interface' % key)
 
         self.contexts[0][macroName(value)] = value
 
     __setitem__ = addGlobal
 
     def addLocal(self, key, value):
-        """ 
-        Add a macro to the local context 
+        """
+        Add a macro to the local context
 
         Required Arguments:
         key -- name of macro to add
@@ -691,20 +701,19 @@ class Context(object):
             instance before being added.
 
         """
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             newvalue = plasTeX.Command()
-            newvalue.unicode = value
+            newvalue.str = value
             value = newvalue
 
         elif not ismacro(value):
-            raise ValueError, \
-                  '"%s" does not implement the macro interface' % key
+            raise ValueError('"%s" does not implement the macro interface' % key)
 
         self.contexts[-1][macroName(value)] = value
 
     def whichCode(self, char):
-        """ 
-        Return the character code that `char` belongs to 
+        """
+        Return the character code that `char` belongs to
 
         Required Arguments:
         char -- character to determine the code of
@@ -722,17 +731,17 @@ class Context(object):
         if char in c[Token.CC_BGROUP]:
             return Token.CC_BGROUP
         if char in c[Token.CC_EGROUP]:
-            return Token.CC_EGROUP 
+            return Token.CC_EGROUP
         if char in c[Token.CC_ESCAPE]:
-            return Token.CC_ESCAPE 
+            return Token.CC_ESCAPE
         if char in c[Token.CC_SUPER]:
             return Token.CC_SUPER
         if char in c[Token.CC_SUB]:
-            return Token.CC_SUB 
+            return Token.CC_SUB
         if char in c[Token.CC_MATHSHIFT]:
             return Token.CC_MATHSHIFT
         if char in c[Token.CC_ALIGNMENT]:
-            return Token.CC_ALIGNMENT 
+            return Token.CC_ALIGNMENT
         if char in c[Token.CC_COMMENT]:
             return Token.CC_COMMENT
         if char in c[Token.CC_ACTIVE]:
@@ -740,7 +749,7 @@ class Context(object):
         if char in c[Token.CC_PARAMETER]:
             return Token.CC_PARAMETER
         if char in c[Token.CC_IGNORED]:
-            return Token.CC_IGNORED 
+            return Token.CC_IGNORED
         if char in c[Token.CC_INVALID]:
             return Token.CC_INVALID
         return Token.CC_OTHER
@@ -748,14 +757,14 @@ class Context(object):
     def catcode(self, char, code):
         """
         Set the category code for a particular character
- 
+
         Required arguments:
         char -- the character to set the code of
         code -- the category code number to set `char` to
 
         """
         c = self.contexts[-1].categories = self.categories = self.categories[:]
-        for i in range(0,16):
+        for i in range(0, 16):
             c[i] = c[i].replace(char, '')
         # Don't insert if it's code 12.
         if code != 12:
@@ -771,8 +780,8 @@ class Context(object):
         self.contexts[-1].categories = self.categories = VERBATIM_CATEGORIES[:]
 
     def newcounter(self, name, resetby=None, initial=0, format=None):
-        """ 
-        Create a new counter 
+        """
+        Create a new counter
 
         This method corresponds to LaTeX's \\newcounter command
 
@@ -786,18 +795,17 @@ class Context(object):
         initial -- initial value for the counter
 
         """
-        name = str(name)
         # Counter already exists
-        if self.counters.has_key(name):
+        if name in list(self.counters.keys()):
             macrolog.debug('counter %s already defined', name)
             return
         self.counters[name] = plasTeX.Counter(self, name, resetby, initial)
 
         if format is None:
             format = '${%s}' % name
-        newclass = new.classobj('the'+name, (plasTeX.TheCounter,), 
+        newclass = type('the' + name, (plasTeX.TheCounter,),
                                {'format': format})
-        self.addGlobal('the'+name, newclass)
+        self.addGlobal('the' + name, newclass)
 
     def newwrite(self, name, file):
         """
@@ -811,80 +819,76 @@ class Context(object):
         self.writes[name] = open(file, 'w')
 
     def newcount(self, name, initial=0):
-        """ 
+        """
         Create a new count (like \\newcount)
 
         Required Arguments:
         name -- name of count to create
-      
+
         Keyword Arguments:
         initial -- value to initialize to
 
         """
-        name = str(name)
         # Generate a new count class
         macrolog.debug('creating count %s', name)
-        newclass = new.classobj(name, (plasTeX.CountCommand,), 
+        newclass = type(name, (plasTeX.CountCommand,),
                                {'value': plasTeX.count(initial)})
         self.addGlobal(name, newclass)
 
     def newdimen(self, name, initial=0):
-        """ 
+        """
         Create a new dimen (like \\newdimen)
 
         Required Arguments:
         name -- name of dimen to create
-      
+
         Keyword Arguments:
         initial -- value to initialize to
 
         """
-        name = str(name)
         # Generate a new dimen class
         macrolog.debug('creating dimen %s', name)
-        newclass = new.classobj(name, (plasTeX.DimenCommand,), 
+        newclass = type(name, (plasTeX.DimenCommand,),
                                 {'value': plasTeX.dimen(initial)})
         self.addGlobal(name, newclass)
 
     def newskip(self, name, initial=0):
-        """ 
+        """
         Create a new glue (like \\newskip)
 
         Required Arguments:
         name -- name of glue to create
-      
+
         Keyword Arguments:
         initial -- value to initialize to
 
         """
-        name = str(name)
         # Generate a new glue class
         macrolog.debug('creating dimen %s', name)
-        newclass = new.classobj(name, (plasTeX.GlueCommand,), 
+        newclass = type(name, (plasTeX.GlueCommand,),
                                 {'value': plasTeX.glue(initial)})
         self.addGlobal(name, newclass)
 
     def newmuskip(self, name, initial=0):
-        """ 
+        """
         Create a new muglue (like \\newmuskip)
 
         Required Arguments:
         name -- name of muglue to create
-      
+
         Keyword Arguments:
         initial -- value to initialize to
 
         """
-        name = str(name)
         # Generate a new muglue class
         macrolog.debug('creating muskip %s', name)
-        newclass = new.classobj(name, (plasTeX.MuGlueCommand,), 
+        newclass = type(name, (plasTeX.MuGlueCommand,),
                                 {'value': plasTeX.muglue(initial)})
         self.addGlobal(name, newclass)
 
     def newif(self, name, initial=False):
-        """ 
-        Create a new \\if (and accompanying) commands 
+        """
+        Create a new \\if (and accompanying) commands
 
         This method corresponds to TeX's \\newif command.
 
@@ -895,37 +899,37 @@ class Context(object):
         Keyword Arguments:
         initial -- initial value of the 'if' command
 
-        """        
-        name = str(name)
+        """
+
         # \if already exists
-        if self.has_key(name):
+        if name in list(self.keys()):
             macrolog.debug('if %s already defined', name)
             return
 
         # Generate new 'if' class
         macrolog.debug('creating if %s', name)
-        ifclass = new.classobj(name, (plasTeX.NewIf,), {'state':initial})
+        ifclass = type(name, (plasTeX.NewIf,), {'state':initial})
         self.addGlobal(name, ifclass)
 
         # Create \iftrue macro
-        truename = name[2:]+'true'
-        newclass = new.classobj(truename, (plasTeX.IfTrue,), {'ifclass':ifclass})
+        truename = name[2:] + 'true'
+        newclass = type(truename, (plasTeX.IfTrue,), {'ifclass':ifclass})
         self.addGlobal(truename, newclass)
 
         # Create \iffalse macro
-        falsename = name[2:]+'false'
-        newclass = new.classobj(falsename, (plasTeX.IfFalse,), {'ifclass':ifclass})
+        falsename = name[2:] + 'false'
+        newclass = type(falsename, (plasTeX.IfFalse,), {'ifclass':ifclass})
         self.addGlobal(falsename, newclass)
 
     def newcommand(self, name, nargs=0, definition=None, opt=None):
-        """ 
-        Create a \\newcommand 
+        """
+        Create a \\newcommand
 
         Required Arguments:
         name -- name of the macro to create
         nargs -- integer number of arguments that the macro has
         definition -- string containing the LaTeX definition
-        opt -- string containing the LaTeX code to use in the 
+        opt -- string containing the LaTeX code to use in the
             optional argument
 
         Examples::
@@ -933,11 +937,9 @@ class Context(object):
             c.newcommand('foo', 2, r'{\\bf #1#2}', opt='myprefix')
 
         """
-        name = str(name)
         # Macro already exists
-        if self.has_key(name):
-            if not issubclass(self[name], (plasTeX.NewCommand, 
-                                           plasTeX.Definition)):
+        if name in list(self.keys()):
+            if not issubclass(self[name], (plasTeX.NewCommand, plasTeX.Definition)):
                 if not issubclass(self[name], plasTeX.TheCounter):
                     return
             macrolog.debug('redefining command "%s"', name)
@@ -945,41 +947,40 @@ class Context(object):
         if nargs is None:
             nargs = 0
         assert isinstance(nargs, int), 'nargs must be an integer'
-        
-        if isinstance(definition, basestring):
+
+        if isinstance(definition, str):
             definition = [x for x in Tokenizer(definition, self)]
-        
-        if isinstance(opt, basestring):
+
+        if isinstance(opt, str):
             opt = [x for x in Tokenizer(opt, self)]
 
         macrolog.debug('creating newcommand %s', name)
-        newclass = new.classobj(name, (plasTeX.NewCommand,),
-                       {'nargs':nargs,'opt':opt,'definition':definition})
+        newclass = type(name, (plasTeX.NewCommand,),
+                       {'nargs':nargs, 'opt':opt, 'definition':definition})
 
         self.addGlobal(name, newclass)
 
-    def newenvironment(self, name, nargs=0, definition=None, opt=None):
-        """ 
-        Create a \\newenvironment 
+    def newenvironment(self, name, nargs=0, def_before=None, def_after=None, opt=None):
+        """
+        Create a \\newenvironment
 
         Required Arguments:
         name -- name of the macro to create
         nargs -- integer number of arguments that the macro has
-        definition -- two-element tuple containing the LaTeX definition.
-            Each element should be a string.  The first element 
-            corresponds to the beginning of the environment, and the
-            second element is the end of the environment.
-        opt -- string containing the LaTeX code to use in the 
+        def_before -- string corresponding to TeX code inserted before the
+            environment
+        def_after -- string corresponding to TeX code inserted after the
+            environment
+        opt -- string containing the LaTeX code to use in the
             optional argument
 
         Examples::
-            c.newenvironment('mylist', 0, (r'\\begin{itemize}', r'\\end{itemize}'))
+            c.newenvironment('mylist', 0, [r'\\begin{itemize}', r'\\end{itemize}'])
 
         """
-        name = str(name)
         # Macro already exists
-        if self.has_key(name):
-            if not issubclass(self[name], (plasTeX.NewCommand, 
+        if name in list(self.keys()):
+            if not issubclass(self[name], (plasTeX.NewCommand,
                                            plasTeX.Definition)):
                 return
             macrolog.debug('redefining environment "%s"', name)
@@ -988,40 +989,35 @@ class Context(object):
             nargs = 0
         assert isinstance(nargs, int), 'nargs must be an integer'
 
-        if definition is not None:
-            assert isinstance(definition, (tuple,list)), \
-                'definition must be a list or tuple'
-            assert len(definition) == 2, 'definition must have 2 elements'
+        if def_before:
+            def_before = list(Tokenizer(def_before, self))
+        if def_after:
+            def_after = list(Tokenizer(def_after, self))
 
-            if isinstance(definition[0], basestring):
-                definition[0] = [x for x in Tokenizer(definition[0], self)]
-            if isinstance(definition[1], basestring):
-                definition[1] = [x for x in Tokenizer(definition[1], self)]
-
-        if isinstance(opt, basestring):
+        if isinstance(opt, str):
             opt = [x for x in Tokenizer(opt, self)]
 
         macrolog.debug('creating newenvironment %s', name)
 
         # Begin portion
-        newclass = new.classobj(name, (plasTeX.NewCommand,),
-                       {'nargs':nargs,'opt':opt,'definition':definition[0]})
+        newclass = type(name, (plasTeX.NewCommand,),
+                       {'nargs':nargs, 'opt':opt, 'definition':def_before})
         self.addGlobal(name, newclass)
 
         # End portion
-        newclass = new.classobj('end'+name, (plasTeX.NewCommand,),
-                       {'nargs':0,'opt':None,'definition':definition[1]})
+        newclass = type('end' + name, (plasTeX.NewCommand,),
+                       {'nargs':0, 'opt':None, 'definition':def_after})
         self.addGlobal('end' + name, newclass)
 
     def newdef(self, name, args=None, definition=None, local=True):
-        """ 
-        Create a \def 
+        """
+        Create a \\def
 
         Required Arguments:
         name -- name of the macro to create
         args -- string containing the TeX argument profile
         definition -- string containing the LaTeX definition
-      
+
         Keyword Arguments:
         local -- indicates whether this macro is local or global
 
@@ -1030,20 +1026,19 @@ class Context(object):
             c.newdef('put', '(#1,#2)#3', '\\dostuff{#1}{#2}{#3}')
 
         """
-        name = str(name)
         # Macro already exists
 #       if self.has_key(name):
-#           if not issubclass(self[name], (plasTeX.NewCommand, 
+#           if not issubclass(self[name], (plasTeX.NewCommand,
 #                                          plasTeX.Definition)):
 #               return
 #           macrolog.debug('redefining definition "%s"', name)
 
-        if isinstance(definition, basestring):
+        if isinstance(definition, str):
             definition = [x for x in Tokenizer(definition, self)]
 
         macrolog.debug('creating def %s', name)
-        newclass = new.classobj(name, (plasTeX.Definition,),
-                       {'args':args,'definition':definition})
+        newclass = type(name, (plasTeX.Definition,),
+                       {'args':args, 'definition':definition})
 
         if local:
             self.addLocal(name, newclass)
@@ -1052,7 +1047,7 @@ class Context(object):
 
     def let(self, dest, source):
         """
-        Create a \let
+        Create a \\let
 
         Required Arguments:
         dest -- the command sequence to create
@@ -1065,17 +1060,31 @@ class Context(object):
         self.lets[dest] = source
 
     def chardef(self, name, num):
-        """ 
+        """
         Create a \\chardef
 
         Required Arguments:
         name -- name of command to create
         num -- character number to use
-      
+
         """
-        name = str(name)
         # Generate a new chardef class
-        macrolog.debug('creating chardef %s', name)
-        newclass = new.classobj(name, (plasTeX.Command,), 
-                                {'unicode':chr(num)})
+        macrolog.debug('creating chardef (8-bit) %s', name)
+        newclass = type(name, (plasTeX.Command,),
+                                {'str':chr(num)})
+        self.addGlobal(name, newclass)
+
+    def mathchardef(self, name, num):
+        """
+        Create a \\mathchardef
+
+        Required Arguments:
+        name -- name of command to create
+        num -- character number to use
+
+        """
+        # Generate a new chardef class
+        macrolog.debug('creating mathchardef (16-bit) %s', name)
+        newclass = type(name, (plasTeX.Command,),
+                                {'str':chr(num)})
         self.addGlobal(name, newclass)
