@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-"""
+r"""
 embed package
 
 To use the embed package, include the following code in your latex document.
@@ -13,9 +13,9 @@ document as-is.
 
 \newif\ifplastex\plastexfalse
 \ifplastex
-    \usepackage{embed}
+    \\usepackage{embed}
 \else
-    \usepackage{comment}\excludecomment{HTML}
+    \\usepackage{comment}\excludecomment{HTML}
     \def\html#1{\def\@html##1#1{}\@html}
 \fi
 
@@ -34,7 +34,6 @@ after
 
 """
 
-from plasTeX import Command, Environment
 from plasTeX.DOM import Text
 from plasTeX.Base.LaTeX.Verbatim import verbatim
 from plasTeX.Base.LaTeX.Verbatim import verb
@@ -45,8 +44,8 @@ class HTML(verbatim):
     
     def digest(self, tokens):
         verbatim.digest(self, tokens)
-        self.unicode = Text(''.join(self))
-        self.unicode.isMarkup = True
+        self.str = Text(''.join(self))
+        self.str.isMarkup = True
         return []
     
 class XHTML(HTML):
@@ -57,9 +56,9 @@ class html(verb):
     
     def digest(self, tokens):
         verb.digest(self, tokens)
-        self.unicode = Text(''.join(self))
-        self.unicode.isMarkup = True
+        self.str = Text(''.join(self))
+        self.str.isMarkup = True
         return []
     
 class xhtml(html):
-    pass
\ No newline at end of file
+    pass
