@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-from plasTeX import Command, IgnoreCommand, Environment
+from plasTeX import Command, IgnoreCommand
 from plasTeX.Base.LaTeX.Verbatim import verbatim
 from plasTeX.DOM import Text
 
@@ -15,8 +15,8 @@ class rawhtml(verbatim):
     blockType = False
     def digest(self, tokens):
         verbatim.digest(self, tokens)
-        self.unicode = Text(''.join(self))
-        self.unicode.isMarkup = True
+        self.str = Text(''.join(self))
+        self.str.isMarkup = True
         return []
         
 class latexonly(IgnoreCommand):
