@@ -1,6 +1,9 @@
 #!/usr/bin/env python
 
-from book import *
+from plasTeX.Packages.book import *
+
+class checkandfixthelayout(Command): pass
+class setlrmarginsandblock(Command): pass
 
 class titleref(Command):
     args = 'label:idref'
@@ -8,7 +11,7 @@ class titleref(Command):
 class tightlist(Command):
     def invoke(self, tex):
         return []
-        
+
 class firmlist(Command):
     def invoke(self, tex):
-        return []
\ No newline at end of file
+        return []
