@@ -5,7 +5,7 @@ subfigure package
 
 """
 
-from plasTeX import Command, Environment
+from plasTeX import Command
 
 def ProcessOptions(options, document):
     context = document.context
@@ -15,10 +15,10 @@ def ProcessOptions(options, document):
                        format='${thetable}.${subtable.alph}')
 
 class subfigurename(Command):
-    unicode = ''
+    str = ''
 
 class subtablename(Command):
-    unicode = ''
+    str = ''
 
 class subfigure(Command):
     args = '[ title ] self'
