@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 
-import new
-from plasTeX.Base import Command, verb
+from plasTeX.Base import Command
 
 class url(Command):
     args = 'url:url'
@@ -16,14 +15,14 @@ class urldef(Command):
 
     def invoke(self, tex):
         Command.invoke(self, tex)
-        name = str(self.attributes['name'])
-        type = str(self.attributes['type'])
+        name = self.attributes['name']
+        type = self.attributes['type']
         c = self.ownerDocument.context
         obj = c[type]()
         obj.parentNode = self.parentNode
         obj.ownerDocument = self.ownerDocument
         result = obj.invoke(tex)
-        c.addGlobal(name, new.classobj(name, (self.DefinedURL,), {'result':result}))
+        c.addGlobal(name, type(name, (self.DefinedURL,), {'result':result}))
 
 class urlstyle(Command):
     args = 'style:str'
@@ -34,4 +33,4 @@ class DeclareUrlCommand(Command):
         Command.invoke(self, tex)
         name = str(self.attributes['name'])
         c = self.ownerDocument.context
-        c.addGlobal(name, new.classobj(name, (url,), {}))
+        c.addGlobal(name, type(name, (url,), {}))
