@@ -10,35 +10,70 @@ support for your own templating engines.
 """
 
 import sys, os, re, plasTeX, shutil, string
+from io import StringIO
 from plasTeX.Renderers import Renderer as BaseRenderer
 from plasTeX.Renderers.PageTemplate.simpletal import simpleTAL, simpleTALES
 from plasTeX.Renderers.PageTemplate.simpletal.simpleTALES import Context as TALContext
-from plasTeX.Renderers.PageTemplate.simpletal.simpleTALUtils import FastStringOutput as StringIO
 
 log = plasTeX.Logging.getLogger()
 
+# Support for Jinja2 templates
+try: 
+    from jinja2 import Environment, contextfunction
+except ImportError:
+    def jinja2template(s, encoding='utf8'):
+        def renderjinja2(obj):
+            return s
+        return renderjinja2
+else:
+    try:
+        import ipdb as pdb
+    except ImportError:
+        import pdb
+        
+    @contextfunction
+    def debug(context):
+        pdb.set_trace()
+
+    def jinja2template(s, encoding='utf8'):
+        env = Environment(trim_blocks=True, lstrip_blocks=True)
+        env.globals['debug'] = debug
+
+        def renderjinja2(obj, s=s):
+            tvars = {'here':obj, 
+                     'obj':obj,
+                     'container':obj.parentNode,
+                     'config':obj.ownerDocument.config,
+                     'context':obj.ownerDocument.context,
+                     'templates':obj.renderer}
+
+            tpl = env.from_string(s)
+            return tpl.render(tvars) 
+
+        return renderjinja2
+
 # Support for Python string templates
-def stringtemplate(s, encoding='utf8'):
-    template = string.Template(unicode(s, encoding))
+def stringtemplate(s):
+    template = string.Template(s)
     def renderstring(obj):
         tvars = {'here':obj, 'self':obj, 'container':obj.parentNode,
                  'config':obj.ownerDocument.config, 'template':template,
                  'templates':obj.renderer, 'context':obj.ownerDocument.context}
-        return unicode(template.substitute(tvars))
+        return template.substitute(tvars)
     return renderstring
 
 # Support for Python string interpolations
-def pythontemplate(s, encoding='utf8'):
+def pythontemplate(s):
     template = s
     def renderpython(obj):
         tvars = {'here':obj, 'self':obj, 'container':obj.parentNode,
                  'config':obj.ownerDocument.config, 'template':template,
                  'templates':obj.renderer, 'context':obj.ownerDocument.context}
-        return unicode(template, encoding) % tvars
+        return template % tvars
     return renderpython
 
 # Support for ZPT HTML and XML templates
-def htmltemplate(s, encoding='utf8'):
+def htmltemplate(s):
     template = simpleTAL.compileHTMLTemplate(s)
     def renderhtml(obj):
         context = TALContext(allowPythonPath=1)
@@ -50,11 +85,11 @@ def htmltemplate(s, encoding='utf8'):
         context.addGlobal('template', template)
         context.addGlobal('templates', obj.renderer)
         output = StringIO()
-        template.expand(context, output, encoding)
-        return unicode(output.getvalue(), encoding)
+        template.expand(context, output)
+        return output.getvalue()
     return renderhtml
 
-def xmltemplate(s, encoding='utf8'):
+def xmltemplate(s):
     template = simpleTAL.compileXMLTemplate(s)
     def renderxml(obj):
         context = TALContext(allowPythonPath=1)
@@ -66,70 +101,69 @@ def xmltemplate(s, encoding='utf8'):
         context.addGlobal('template', template)
         context.addGlobal('templates', obj.renderer)
         output = StringIO()
-        template.expand(context, output, encoding, docType=None, suppressXMLDeclaration=1)
-        return unicode(output.getvalue(), encoding)
+        template.expand(context, output, docType=None, suppressXMLDeclaration=1)
+        return output.getvalue()
     return renderxml
 
 # Support for Cheetah templates
-try: 
+try:
 
-    from Cheetah.Template import Template as CheetahTemplate
-    from Cheetah.Filters import Filter as CheetahFilter
-    class CheetahUnicode(CheetahFilter):
-        def filter(self, val, encoding='utf-8', **kw):
-            return unicode(val).encode(encoding)
-    def cheetahtemplate(s, encoding='utf8'):
-        def rendercheetah(obj, s=s):
+   from Cheetah.Template import Template as CheetahTemplate
+   from Cheetah.Filters import Filter as CheetahFilter
+   class CheetahUnicode(CheetahFilter):
+       def filter(self, val, **kw):
+           return str(val)
+   def cheetahtemplate(s):
+       def rendercheetah(obj, s=s):
+           tvars = {'here':obj, 'container':obj.parentNode,
+                    'config':obj.ownerDocument.config,
+                    'context':obj.ownerDocument.context,
+                    'templates':obj.renderer}
+           return CheetahTemplate(source=s, searchList=[tvars],
+                                  filter=CheetahUnicode).respond()
+       return rendercheetah
+
+except (ImportError, AttributeError):
+
+   def cheetahtemplate(s):
+       def rendercheetah(obj):
+           return str(s)
+       return rendercheetah
+
+# Support for Kid templates
+try:
+
+    from kid import Template as KidTemplate
+
+    def kidtemplate(s):
+        # Add namespace py: in
+        s = '<div xmlns:py="http://purl.org/kid/ns#" py:strip="True">%s</div>' % s
+        def renderkid(obj, s=s):
             tvars = {'here':obj, 'container':obj.parentNode,
                      'config':obj.ownerDocument.config,
                      'context':obj.ownerDocument.context,
                      'templates':obj.renderer}
-            return CheetahTemplate(source=s, searchList=[tvars], 
-                                   filter=CheetahUnicode).respond()
-        return rendercheetah
-
-except ImportError:
-
-    def cheetahtemplate(s, encoding='utf8'):
-        def rendercheetah(obj):
-            return unicode(s, encoding)
-        return rendercheetah
-
-# Support for Kid templates
-try: 
-
-    from kid import Template as KidTemplate
-
-    def kidtemplate(s, encoding='utf8'):
-        # Add namespace py: in
-        s = '<div xmlns:py="http://purl.org/kid/ns#" py:strip="True">%s</div>' % s
-        def renderkid(obj, s=s):
-            tvars = {'here':obj, 'container':obj.parentNode, 
-                     'config':obj.ownerDocument.config,
-                     'context':obj.ownerDocument.context,
-                     'templates':obj.renderer}
-            return unicode(KidTemplate(source=s, 
-                   **tvars).serialize(encoding=encoding, fragment=1), encoding)
+            return str(KidTemplate(source=s, **tvars).serialize(fragment=1))
         return renderkid
 
 except ImportError:
 
-    def kidtemplate(s, encoding='utf8'):
+    def kidtemplate(s):
         def renderkid(obj):
-            return unicode(s, encoding)
+            return str(s)
         return renderkid
 
 # Support for Genshi templates
-try: 
+try:
 
     from genshi.template import MarkupTemplate as GenshiTemplate
     from genshi.template import TextTemplate as GenshiTextTemplate
     from genshi.core import Markup
 
     def markup(obj):
-        return Markup(unicode(obj))
+        return Markup(str(obj))
 
-    def genshixmltemplate(s, encoding='utf8'):
+    def genshixmltemplate(s):
         # Add namespace py: in
         s = '<div xmlns:py="http://genshi.edgewall.org/" py:strip="True">%s</div>' % s
         template = GenshiTemplate(s)
@@ -138,11 +172,10 @@ try:
                      'config':obj.ownerDocument.config, 'template':template,
                      'context':obj.ownerDocument.context,
                      'templates':obj.renderer}
-            return unicode(template.generate(**tvars).render(method='xml', 
-                           encoding=encoding), encoding)
+            return str(template.generate(**tvars).render(method='xml'))
         return rendergenshixml
 
-    def genshihtmltemplate(s, encoding='utf8'):
+    def genshihtmltemplate(s):
         # Add namespace py: in
         s = '<div xmlns:py="http://genshi.edgewall.org/" py:strip="True">%s</div>' % s
         template = GenshiTemplate(s)
@@ -151,42 +184,40 @@ try:
                      'config':obj.ownerDocument.config, 'template':template,
                      'context':obj.ownerDocument.context,
                      'templates':obj.renderer}
-            return unicode(template.generate(**tvars).render(method='html', 
-                           encoding=encoding), encoding)
+            return str(template.generate(**tvars).render(method='html'))
         return rendergenshihtml
 
-    def genshitexttemplate(s, encoding='utf8'):
+    def genshitexttemplate(s):
         template = GenshiTextTemplate(s)
         def rendergenshitext(obj):
             tvars = {'here':obj, 'container':obj.parentNode, 'markup':markup,
                      'config':obj.ownerDocument.config, 'template':template,
                      'context':obj.ownerDocument.context,
                      'templates':obj.renderer}
-            return unicode(template.generate(**tvars).render(method='text', 
-                           encoding=encoding), encoding)
+            return str(template.generate(**tvars).render(method='text'))
         return rendergenshitext
 
 except ImportError:
 
-    def genshixmltemplate(s, encoding='utf8'):
+    def genshixmltemplate(s):
         def rendergenshixml(obj):
-            return unicode(s, encoding)
+            return str(s)
         return rendergenshixml
 
-    def genshihtmltemplate(s, encoding='utf8'):
+    def genshihtmltemplate(s):
         def rendergenshihtml(obj):
-            return unicode(s, encoding)
+            return str(s)
         return rendergenshihtml
 
-    def genshitexttemplate(s, encoding='utf8'):
+    def genshitexttemplate(s):
         def rendergenshitext(obj):
-            return unicode(s, encoding)
+            return str(s)
         return rendergenshitext
 
 
 def copytree(src, dest, symlink=None):
-    """ 
-    This is the same as shutil.copytree, but doesn't error out if the 
+    """
+    This is the same as shutil.copytree, but doesn't error out if the
     directories already exist.
 
     """
@@ -202,10 +233,10 @@ def copytree(src, dest, symlink=None):
                 os.symlink(os.readlink(srcpath), destpath)
             elif not os.path.isdir(destpath):
                 os.makedirs(destpath)
-                try: 
+                try:
                     shutil.copymode(srcpath, destpath)
                 except: pass
-                try: 
+                try:
                     shutil.copystat(srcpath, destpath)
                 except: pass
         for f in files:
@@ -232,12 +263,12 @@ class TemplateEngine(object):
 class PageTemplate(BaseRenderer):
     """ Renderer for page template based documents """
 
-    outputType = unicode
+    outputType = str
     fileExtension = '.xml'
-    encodingErrors = 'xmlcharrefreplace'
 
     def __init__(self, *args, **kwargs):
         BaseRenderer.__init__(self, *args, **kwargs)
+        self.loadedTheme = None
         self.engines = {}
         htmlexts = ['.html','.htm','.xhtml','.xhtm','.zpt','.pt']
         self.registerEngine('pt', None, htmlexts, htmltemplate)
@@ -254,6 +285,7 @@ class PageTemplate(BaseRenderer):
         self.registerEngine('genshi', None, '.gen', genshihtmltemplate)
         self.registerEngine('genshi', 'xml', '.genx', genshixmltemplate)
         self.registerEngine('genshi', 'text', '.gent', genshitexttemplate)
+        self.registerEngine('jinja2', None, '.jinja2', jinja2template)
 
     def registerEngine(self, name, type, ext, function):
         """
@@ -273,8 +305,8 @@ class PageTemplate(BaseRenderer):
         self.engines[key] = TemplateEngine(ext, function)
 
     def textDefault(self, node):
-        """ 
-        Default renderer for text nodes 
+        """
+        Default renderer for text nodes
 
         This method makes sure that special characters are converted to
         entities.
@@ -283,6 +315,7 @@ class PageTemplate(BaseRenderer):
         node -- the Text node to process
 
         """
+
         if not(getattr(node, 'isMarkup', None)):
             node = node.replace('&', '&amp;')
             node = node.replace('<', '&lt;')
@@ -293,7 +326,7 @@ class PageTemplate(BaseRenderer):
         """ Load and compile page templates """
         themename = document.config['general']['theme']
 
-        # Load templates from renderer directory and parent 
+        # Load templates from renderer directory and parent
         # renderer directories
         sup = list(type(self).__mro__)
         sup.reverse()
@@ -320,6 +353,7 @@ class PageTemplate(BaseRenderer):
             if os.path.isdir(theme):
                 log.info('Importing templates from %s' % theme)
                 self.importDirectory(theme)
+                self.loadedTheme = theme
 
                 extensions = []
                 for e in self.engines.values():
@@ -346,19 +380,19 @@ class PageTemplate(BaseRenderer):
         BaseRenderer.render(self, document)
 
     def importDirectory(self, templatedir):
-        """ 
-        Compile all ZPT files in the given directory 
+        """
+        Compile all ZPT files in the given directory
 
         Templates can exist in two different forms.  First, a template
-        can be a file unto itself.  If an XML template is desired, 
+        can be a file unto itself.  If an XML template is desired,
         the file should have an extension of .xml, .xhtml, or .xhtm.
-        If an HTML template is desired, the files should have an 
-        extension of .zpt, .html, or .htm.  You can also configure 
+        If an HTML template is desired, the files should have an
+        extension of .zpt, .html, or .htm.  You can also configure
         your own page templates with their own extensions.
 
         If you have many small templates, or a template that corresponds
         to more than one macro, you can use a multiple ZPT file.  A
-        multiple ZPT file contains directives within it to delimit 
+        multiple ZPT file contains directives within it to delimit
         individual page templates as well as specify which macros they
         correspond to and what type of template they are (i.e. XML or
         HTML).
@@ -369,20 +403,20 @@ class PageTemplate(BaseRenderer):
         """
         # Create a list for resolving aliases
         self.aliases = {}
-        
+
         enames = {}
         for key, value in self.engines.items():
             for i in value.ext:
                 enames[i+'s'] = key[0]
-                
+
         singleenames = {}
         for key, value in self.engines.items():
             for i in value.ext:
                 singleenames[i] = key[0]
-                
+
         if templatedir and os.path.isdir(templatedir):
             files = os.listdir(templatedir)
-            
+
             # Compile multi-pt files first
             for f in files:
                 ext = os.path.splitext(f)[-1]
@@ -409,21 +443,21 @@ class PageTemplate(BaseRenderer):
                 for value in self.engines.values():
                     if ext in value.ext:
                         options['engine'] = singleenames[ext.lower()]
-                        self.parseTemplates(f, options)                
+                        self.parseTemplates(f, options)
                         del options['engine']
                         break
 
         if self.aliases:
-           log.warning('The following aliases were unresolved: %s' 
-                       % ', '.join(self.aliases.keys())) 
+           log.warning('The following aliases were unresolved: %s'
+                       % ', '.join(list(self.aliases.keys())))
 
     def setTemplate(self, template, options):
-        """ 
-        Compile template and set it in the renderer 
+        """
+        Compile template and set it in the renderer
 
         Required Arguments:
         template -- the content of the template to be compiled
-        options -- dictionary containing the name (or names) and type 
+        options -- dictionary containing the name (or names) and type
             of the template
 
         """
@@ -434,9 +468,9 @@ class PageTemplate(BaseRenderer):
             if not names:
                 names = [' ']
         except KeyError:
-            raise ValueError, 'No name given for template'
+            raise ValueError('No name given for template')
 
-        # If an alias was specified, link the names to the 
+        # If an alias was specified, link the names to the
         # already specified template.
         if 'alias' in options:
             alias = options['alias'].strip()
@@ -446,7 +480,7 @@ class PageTemplate(BaseRenderer):
                 log.warning('Both an alias and a template were specified for: %s' % ', '.join(names))
 
         # Resolve remaining aliases
-        for key, value in self.aliases.items():
+        for key, value in list(self.aliases.items()):
             if value in self:
                 self[key] = self[value]
             self.aliases.pop(key)
@@ -461,14 +495,13 @@ class PageTemplate(BaseRenderer):
             ttype = ttype.lower()
         engine = options.get('engine','zpt').lower()
 
-        templateeng = self.engines.get((engine, ttype), 
+        templateeng = self.engines.get((engine, ttype),
                             self.engines.get((engine, None)))
-     
+
         try:
             template = templateeng.compile(template)
-        except Exception, msg:
-#           print msg
-            raise ValueError, 'Could not compile template "%s"' % names[0]
+        except Exception as msg:
+            raise ValueError('Could not compile template "%s"' % names[0])
 
         for name in names:
             self[name] = template
@@ -485,9 +518,10 @@ class PageTemplate(BaseRenderer):
             in the file
 
         """
+        num_templates = 0
         template = []
         options = options.copy()
-        defaults = {}
+        defaults = options.copy()
         name = None
         if not options or 'name' not in options:
             f = open(filename, 'r')
@@ -498,12 +532,13 @@ class PageTemplate(BaseRenderer):
                     # Purge any awaiting templates
                     if template:
                         try:
+                            num_templates += 1
                             self.setTemplate(''.join(template), options)
-                        except ValueError, msg:
-                            print 'ERROR: %s at line %s in file %s' % (msg, i, filename)
+                        except ValueError as msg:
+                            print('ERROR: %s at line %s in file %s' % (msg, i, filename))
                         options = defaults.copy()
                         template = []
-    
+
                     # Done purging previous template, start a new one
                     name, value = line.split(':', 1)
                     name = name.strip()
@@ -513,39 +548,40 @@ class PageTemplate(BaseRenderer):
                         for line in f:
                             value += line.rstrip()
                             break
-    
+
                     value = re.sub(r'\s+', r' ', value.strip())
                     if name.startswith('default-'):
                         name = name.split('-')[-1]
                         defaults[name] = value
-                        if name not in options:
+                        if name not in options or num_templates == 0:
                             options[name] = value
                     else:
                         options[name] = value
                     continue
-                
+
                 if template or (not(template) and line.strip()):
                     template.append(line)
                 elif not(template) and 'name' in options:
                     template.append('')
 
         else:
-            template = open(filename, 'r').readlines()
-    
+            with open(filename, encoding='utf-8') as f:
+                template = f.readlines()
+
         # Purge any awaiting templates
         if template:
             try:
                 self.setTemplate(''.join(template), options)
-            except ValueError, msg:
-                print 'ERROR: %s in template %s in file %s' % (msg, ''.join(template), filename)
+            except ValueError as msg:
+                print('ERROR: %s in template %s in file %s' % (msg, ''.join(template), filename))
 
         elif name and not(template):
             self.setTemplate('', options)
 
     def processFileContent(self, document, s):
         # Add width, height, and depth to images
-        s = re.sub(r'&amp;(\S+)-(width|height|depth);(?:&amp;([a-z]+);)?', 
-                   self.setImageData, s) 
+        s = re.sub(r'&amp;(\S+)-(width|height|depth);(?:&amp;([a-z]+);)?',
+                   self.setImageData, s)
 
         # Convert characters >127 to entities
         if document.config['files']['escape-high-chars']:
@@ -553,10 +589,10 @@ class PageTemplate(BaseRenderer):
             for i, item in enumerate(s):
                 if ord(item) > 127:
                     s[i] = '&#%.3d;' % ord(item)
-            s = u''.join(s)
-
+            s = ''.join(s)
+        
         return BaseRenderer.processFileContent(self, document, s)
-             
+
     def setImageData(self, m):
         """
         Substitute in width, height, and depth parameters in image tags
@@ -564,7 +600,7 @@ class PageTemplate(BaseRenderer):
         The width, height, and depth parameters aren't known until after
         all of the output has been generated.  We have to post-process
         the files to insert this information.  This method replaces
-        the &filename-width;, &filename-height;, and &filename-depth; 
+        the &filename-width;, &filename-height;, and &filename-depth;
         placeholders with their appropriate values.
 
         Required Arguments:
