@@ -7,10 +7,10 @@ import textwrap, re, string
 class TextRenderer(BaseRenderer):
     """ Renderer for plain text documents """
 
-    outputType = unicode
+    outputType = str
     fileExtension = '.txt'
     lineWidth = 76
-    
+
     aliases = {
         'superscript': 'active::^',
         'subscript': 'active::_',
@@ -26,10 +26,10 @@ class TextRenderer(BaseRenderer):
         'at': '@',
         'backslash': '\\',
     }
-    
+
     def __init__(self, *args, **kwargs):
         BaseRenderer.__init__(self, *args, **kwargs)
-        
+
         # Load dictionary with methods
         for key in dir(self):
             if key.startswith('do__'):
@@ -38,7 +38,7 @@ class TextRenderer(BaseRenderer):
                 self[key[3:]] = getattr(self, key)
 
         self['default-layout'] = self['document-layout'] = self.default
-        
+
         self.footnotes = []
         self.blocks = []
 
@@ -53,9 +53,9 @@ class TextRenderer(BaseRenderer):
         # Handle characters like \&, \$, \%, etc.
         if len(node.nodeName) == 1 and node.nodeName not in encoding.stringletters():
             return self.textDefault(node.nodeName)
-            
+
         # Render child nodes
-        return unicode(node)
+        return str(node)
 
     def processFileContent(self, document, s):
         s = BaseRenderer.processFileContent(self, document, s)
@@ -77,39 +77,39 @@ class TextRenderer(BaseRenderer):
                 before = '\n'.join(spaces) + '\n'
 
             block = self.blocks[int(m.group(2))]
-            block = space + block.replace('\n', u'\n%s' % space) 
+            block = space + block.replace('\n', u'\n%s' % space)
 
             s = block_re.sub('%s%s' % (before, block), s, 1)
 
         # Clean up newlines
         return re.sub(r'\s*\n\s*\n(\s*\n)+', r'\n\n\n', s)
-    
+
     def textDefault(self, node):
-        return unicode(node)
+        return str(node)
 
     def wrap(self, s, **kwargs):
-        return textwrap.wrap(unicode(s), self.lineWidth, break_long_words=False, **kwargs)
-    
+        return textwrap.wrap(str(s), self.lineWidth, break_long_words=False, **kwargs)
+
     def fill(self, s, **kwargs):
-        return textwrap.fill(unicode(s), self.lineWidth, break_long_words=False, **kwargs)
-    
+        return textwrap.fill(str(s), self.lineWidth, break_long_words=False, **kwargs)
+
     # Alignment
 
     def do_flushleft(self, node):
-        return self.fill(unicode(node)).strip()
-        
+        return self.fill(str(node)).strip()
+
     do_raggedbottom = do_raggedright = do_leftline = do_flushleft
-    
+
     def center(self, text):
         s = self.wrap(text)
         for i, line in enumerate(s):
             indent = int((self.lineWidth - len(line)) / 2)
             s[i] = '%s%s' % (' '*indent, line)
         return '\n'.join(s)
-            
+
     def do_center(self, node):
-        return self.center(unicode(node))
-        
+        return self.center(str(node))
+
     do_centering = do_centerline = do_center
 
     def do_flushright(self, node):
@@ -118,17 +118,17 @@ class TextRenderer(BaseRenderer):
             indent = self.lineWidth - len(line)
             s[i] = '%s%s' % (' '*indent, line)
         return '\n'.join(s).rstrip()
-        
+
     do_raggedleft = do_llap = do_flushright
-    
+
     # Arrays
-    
-    def do_array(self, node, cellspacing=(2,1), render=unicode):
+
+    def do_array(self, node, cellspacing=(2,1), render=str):
         # Render the table cells and get min and max column widths
         colwidths = []
         for r, row in enumerate(node):
             for c, cell in enumerate(row):
-                if isinstance(render, basestring):
+                if isinstance(render, str):
                     s = getattr(cell, render)().strip()
                 else:
                     s = render(cell).strip()
@@ -140,7 +140,7 @@ class TextRenderer(BaseRenderer):
                 if r == 0:
                     colwidths.append([minlength, maxlength])
                 else:
-                    colwidths[c] = [max(minlength, colwidths[c][0]), 
+                    colwidths[c] = [max(minlength, colwidths[c][0]),
                                     max(maxlength, colwidths[c][1])]
 
         # Determine best column widths
@@ -175,7 +175,7 @@ class TextRenderer(BaseRenderer):
             for c, cell in enumerate(row):
                 origwidth = self.lineWidth
                 self.lineWidth = outwidths[c]
-                if isinstance(render, basestring):
+                if isinstance(render, str):
                     s = getattr(cell, render)().split('\n')
                 else:
                     s = render(cell).strip().split('\n')
@@ -196,7 +196,7 @@ class TextRenderer(BaseRenderer):
                 for i, line in enumerate(content):
                     content[i] = content[i] + ' '*(outwidths[c]-width+cellspacing[0])
                 rendered[r][c] = content
-        
+
         # Piece all of the table parts together
         output = []
         for row in rendered:
@@ -207,16 +207,16 @@ class TextRenderer(BaseRenderer):
                 # Get rid of unneeded whitespace and put in a line break
                 output[-1] = output[-1].strip()
                 output.append('\n')
-            
+
         return ''.join(output)
-        
+
     do_tabular = do_tabularx = do_longtable = do_array
-    
+
     def do_cline(self, node):
         return ''
-        
+
     def do_multicolumn(self, node):
-        return unicode(node)
+        return str(node)
 
     # Bibliography
     def do_thebibliography(self, node):
@@ -228,39 +228,39 @@ class TextRenderer(BaseRenderer):
                                         subsequent_indent=' '*bulletlen))
             output.append('')
         output.append('')
-        return u'\n'.join(output)        
+        return u'\n'.join(output)
 
     def do_bibliographystyle(self, node):
         return u''
 
     def do_bibliography(self, node):
         return self.default(node)
-  
+
     def do_cite(self, node):
         output = []
         for item in node.citation():
-            output.append(unicode(item))
+            output.append(str(item))
         return u''.join(output)
 
     def do_bibliographyref(self, node):
         return self.default(node)
 
     # Boxes
-    
+
     do_mbax = do_makebox = do_fbox = do_framebox = do_parbox = default
     do_minipage = do_raisebox = do_rule = default
-    
+
     # Breaking
-    
+
     def do_linebreak(self, node):
         return u'\n\n'
-        
+
     do_newline = do_pagebreak = do_newpage = do_clearpage = do_cleardoublepage = do_linebreak
 
     # Crossref
-    
+
     def do_ref(self, node):
-        return unicode(node.idref['label'].ref)
+        return str(node.idref['label'].ref)
 
     def do_pageref(self, node):
         return u'*'
@@ -269,17 +269,17 @@ class TextRenderer(BaseRenderer):
         return u''
 
     # Floats
-    
+
     def do_figure(self, node):
-        return unicode(node)
-        
+        return str(node)
+
     do_table = do_marginpar = do_figure
 
     def do_caption(self, node):
-        return u'%s %s: %s' % (node.title, node.ref, unicode(node))
+        return u'%s %s: %s' % (node.title, node.ref, str(node))
 
     # Font Selection
-    
+
     do_mdseries = do_textmd = do_bfseries = do_textbf = default
     do_rmfamily = do_textrm = do_sffamily = do_textsf = default
     do_ttfamily = do_texttt = do_upshape = do_textup = default
@@ -298,21 +298,21 @@ class TextRenderer(BaseRenderer):
         self.footnotes.append(self.fill(node, initial_indent='%s ' % mark,
                                      subsequent_indent=' ' * (len(mark)+1)).strip())
         return mark
-        
+
     def do_footnotetext(self, node):
         self.do_footnote(self, node)
         return ''
-    
+
     def do_footnotemark(self, node):
         return u'[%s]' % (len(self.footnotes)+1)
-    
+
     # Index
-    
+
     def do_theindex(self, node):
         return u''
-        
+
     do_printindex = do_index = do_theindex
-    
+
     # Lists
 
     def do_itemize(self, node):
@@ -323,7 +323,7 @@ class TextRenderer(BaseRenderer):
             output.append(self.fill(item, initial_indent=bullet,
                                         subsequent_indent=' '*bulletlen))
         return self.addBlock(u'\n'.join(output))
-        
+
     def do_enumerate(self, node):
         output = []
         for i, item in enumerate(node):
@@ -332,7 +332,7 @@ class TextRenderer(BaseRenderer):
             output.append(self.fill(item, initial_indent=bullet,
                                         subsequent_indent=' '*bulletlen))
         return self.addBlock(u'\n'.join(output))
-        
+
     def do_description(self, node):
         output = []
         for item in node:
@@ -343,20 +343,20 @@ class TextRenderer(BaseRenderer):
         return self.addBlock(u'\n'.join(output))
 
     do_list = do_trivlist = do_description
-    
+
     # Math
-    
+
     def do_math(self, node):
         return re.sub(r'\s*(_|\^)\s*', r'\1', node.source)
 
     do_ensuremath = do_math
-    
+
     def do_equation(self, node):
         s = u'   %s' % re.compile(r'^\s*\S+\s*(.*?)\s*\S+\s*$', re.S).sub(r'\1', node.source)
         return re.sub(r'\s*(_|\^)\s*', r'\1', s)
 
     do_displaymath = do_equation
-    
+
     def do_eqnarray(self, node):
         def render(node):
             s = re.compile(r'^\$\\displaystyle\s*(.*?)\s*\$\s*$', re.S).sub(r'\1', node.source)
@@ -365,18 +365,18 @@ class TextRenderer(BaseRenderer):
         output = []
         for line in s.split('\n'):
             output.append('   %s' % line)
-        return u'\n'.join(output)     
+        return u'\n'.join(output)
 
-    do_align = do_gather = do_falign = do_multiline = do_eqnarray 
+    do_align = do_gather = do_falign = do_multiline = do_eqnarray
     do_multline = do_alignat = do_split = do_eqnarray
-    
+
     # Misc
-    
+
     do_bgroup = default
-    
+
     def do_def(self, node):
         return u''
-    
+
     do_tableofcontents = do_input = do_protect = do_let = do_def
     do_newcommand = do_hfill = do_hline = do_openout = do_renewcommand = do_def
     do_write = do_hspace = do_appendix = do_global = do_noindent = do_def
@@ -387,21 +387,21 @@ class TextRenderer(BaseRenderer):
 
     def do_egroup(self, node):
         return u''
-    
+
     # Pictures
-    
+
     def do_picture(self, node):
         return u''
-        
+
     # Primitives
-    
+
     def do_par(self, node):
         numchildren = len(node.childNodes)
-        if numchildren == 1 and not isinstance(node[0], basestring):
-            return u'%s\n\n' % unicode(node)
-        elif numchildren == 2 and isinstance(node[1], basestring) and \
+        if numchildren == 1 and not isinstance(node[0], str):
+            return u'%s\n\n' % str(node)
+        elif numchildren == 2 and isinstance(node[1], str) and \
            not node[1].strip():
-            return u'%s\n\n' % unicode(node)
+            return u'%s\n\n' % str(node)
         s = u'%s\n\n' % self.fill(node)
         if not s.strip():
             return u''
@@ -409,36 +409,36 @@ class TextRenderer(BaseRenderer):
 
     def do__superscript(self, node):
         return self.default(node)
-    
+
     def do__subscript(self, node):
         return self.default(node)
-    
+
     # Quotations
-    
+
     def do_quote(self, node):
         backslash = self['\\']
         self['\\'] = lambda *args: u'\001'
-        res = [x.strip() for x in unicode(node).split(u'\001')]
+        res = [x.strip() for x in str(node).split(u'\001')]
         output = []
-        for par in [x.strip() for x in unicode(node).split(u'\n\n')]:
+        for par in [x.strip() for x in str(node).split(u'\n\n')]:
             for item in [x.strip() for x in par.split(u'\001')]:
                 output.append(self.fill(item, initial_indent='   ', subsequent_indent='      '))
             output.append('')
         output.pop()
         self['\\'] = backslash
         return u'\n'.join(output)
-    
+
     do_quotation = do_verse = do_quote
 
     # Sectioning
 
     def do_document(self, node):
-        content = unicode(node).rstrip()
+        content = str(node).rstrip()
         footnotes = u'\n\n'.join(self.footnotes).rstrip()
         if footnotes:
             content = u'%s\n\n\n%s' % (content, footnotes)
         return u'%s\n\n' % content
-        
+
     def do_maketitle(self, node):
         output = []
         metadata = node.ownerDocument.userdata
@@ -470,9 +470,9 @@ class TextRenderer(BaseRenderer):
 
     def do_title(self, node):
         return u''
-        
+
     do_author = do_date = do_thanks = do_title
-    
+
     def do_abstract(self, node):
         return self.center(node)
 
@@ -480,25 +480,25 @@ class TextRenderer(BaseRenderer):
 
     def do__dollar(self, node):
         return u'$'
-        
+
     def do__percent(self, node):
         return u'%'
-        
+
     def do__opencurly(self, node):
         return u'{'
-        
+
     def do__closecurly(self, node):
         return u'}'
-    
+
     def do__underscore(self, node):
         return u'_'
-        
+
     def do__ampersand(self, node):
         return u'&'
-        
+
     def do__hashmark(self, node):
         return u'#'
-    
+
     def do__space(self, node):
         return u' '
 
@@ -510,7 +510,7 @@ class TextRenderer(BaseRenderer):
 
     def do_emph(self, node):
         return self.default(node)
-        
+
     do_em = do_emph
 
     def do__tilde(self, node):
@@ -520,7 +520,7 @@ class TextRenderer(BaseRenderer):
         return u' '
 
     do_quad = do_qquad = do_enspace
-        
+
     def do_enskip(self, node):
         return u''
 
@@ -530,21 +530,21 @@ class TextRenderer(BaseRenderer):
         return self.default(node)
 
     # Space
-    
+
     def do_hspace(self, node):
         return u' '
-    
+
     def do_vspace(self, node):
         return u''
-        
+
     do_bigskip = do_medskip = do_smallskip = do_vspace
 
     # Tabbing - not implemented yet
-    
+
     # Verbatim
-    
+
     def do_verbatim(self, node):
-        return re.sub(r'^\s*\n', r'', unicode(node)).rstrip()
+        return re.sub(r'^\s*\n', r'', str(node)).rstrip()
 
     do_alltt = do_verbatim
 
