@@ -1,10 +1,11 @@
 #!/usr/bin/env python
 
-import codecs, os, shutil, string
+import os, shutil, string, importlib
 from plasTeX.Filenames import Filenames
 from plasTeX.DOM import Node
 from plasTeX.Logging import getLogger
 from plasTeX.Imagers import Image, PILImage
+import collections.abc
 
 log = getLogger()
 status = getLogger('status')
@@ -19,7 +20,7 @@ def baseclasses(cls):
     output = [cls]
     for item in cls.__bases__:
         output.extend(baseclasses(item))
-    return [x for x in output if x is not object] 
+    return [x for x in output if x is not object]
 
 def mixin(base, mix, overwrite=False):
     """
@@ -30,21 +31,21 @@ def mixin(base, mix, overwrite=False):
     mix -- the mixin class
 
     """
-    if not vars(base).has_key('_mixed_'):
+    if '_mixed_' not in vars(base):
         base._mixed_ = {}
     mixed = base._mixed_
     for cls in baseclasses(mix):
-        for item, value in vars(cls).items():
+        for item, value in list(vars(cls).items()):
             if item in ['__dict__','__module__','__doc__','__weakref__']:
                 continue
-            if overwrite or not vars(base).has_key(item):
+            if overwrite or item not in vars(base):
                 old = vars(base).get(item, None)
                 setattr(base, item, value)
                 mixed[item] = (mix, old)
 
 def unmix(base, mix=None):
     """
-    Remove mixed in methods and members 
+    Remove mixed in methods and members
 
     Required Arguments:
     base -- the base class to remove mixins from
@@ -54,14 +55,14 @@ def unmix(base, mix=None):
 
     """
     if mix is None:
-        for key, value in base._mixed_.items():
+        for key, value in list(base._mixed_.items()):
             if value[1] is not None:
                 setattr(base, key, value[1])
             else:
                 delattr(base, key)
         del base._mixed_
     else:
-        for key, value in base._mixed_.items():
+        for key, value in list(base._mixed_.items()):
             if value[0] is mix:
                 if value[1] is not None:
                     setattr(base, key, value[1])
@@ -75,12 +76,12 @@ class Renderable(object):
     """
     Base class for all renderable nodes
 
-    This class is mixed into nodes of the document object prior to 
-    rendering.  The actual rendering method is __unicode__.
+    This class is mixed into nodes of the document object prior to
+    rendering.  The actual rendering method is __str__.
 
     """
 
-    def __unicode__(self):
+    def __str__(self):
         """
         Invoke the rendering process on all of the child nodes.
 
@@ -88,22 +89,22 @@ class Renderable(object):
         r = Node.renderer
 
         # Short circuit macros that have unicode equivalents
-        uni = self.unicode
-        if uni is not None: 
+        uni = self.str
+        if uni is not None:
             return r.outputType(r.textDefault(uni))
 
         # If we don't have childNodes, then we're done
         if not self.hasChildNodes():
 #           if self.filename:
 #               status.info(' [ %s ] ', self.filename)
-            return u''
+            return ''
 
 #       if self.filename:
 #           status.info(' [ %s ', self.filename)
 
         # At the very top level, only render the DOCUMENT_LEVEL node
         if self.nodeType == Node.DOCUMENT_NODE:
-            childNodes = [x for x in self.childNodes 
+            childNodes = [x for x in self.childNodes
                             if x.level == Node.DOCUMENT_LEVEL]
         else:
             childNodes = self.childNodes
@@ -118,8 +119,8 @@ class Renderable(object):
                 continue
 
             # Short circuit macros that have unicode equivalents
-            uni = child.unicode
-            if uni is not None: 
+            uni = child.str
+            if uni is not None:
                 s.append(r.textDefault(uni))
                 continue
 
@@ -127,42 +128,54 @@ class Renderable(object):
             nodeName = child.nodeName
             modifier = None
 
+            # Does the macro specify an alternative templateName
+            templateName = getattr(child, 'templateName', None)
             # Does the macro have a modifier (i.e. '*')
             if child.attributes:
                 modifier = child.attributes.get('*modifier*')
 
             if child.filename:
-            
                 # Force footnotes to be cached
                 if hasattr(child, 'footnotes'):
-                    child.footnotes
+                    _ = child.footnotes
 
                 status.info(' [ %s ', child.filename)
 
+                # Filename and templateName
+                if templateName:
+                    layouts.append('%s-layout' % (templateName))
+                    if modifier:
+                        layouts.append('%s-layout%s' % (templateName, modifier))
+
                 # Filename and modifier
                 if modifier:
                     layouts.append('%s-layout%s' % (nodeName, modifier))
 
-                # Filename only
+                # Add nodeName to list
                 layouts.append('%s-layout' % nodeName)
 
-            # Modifier only
+            # templateName
+            if templateName:
+                names.append(templateName)
+                if modifier:
+                    names.append('%s%s' % (templateName, modifier))
+            # Modifier
             if modifier:
                 names.append('%s%s' % (nodeName, modifier))
 
             names.append(nodeName)
             layouts.append('default-layout')
 
-            # Locate the rendering callable, and call it with the 
+            # Locate the rendering callable, and call it with the
             # current object (i.e. `child`) as its argument.
             func = r.find(names, r.default)
             val = func(child)
 
-            # If a plain string is returned, we have no idea what 
+            # If a plain string is returned, we have no idea what
             # the encoding is, but we'll make a guess.
-            if type(val) is not unicode:
+            if type(val) is not str:
                 log.warning('The renderer for %s returned a non-unicode string.  Using the default input encoding.' % type(child).__name__)
-                val = unicode(val, child.config['files']['input-encoding'])
+                val = str(val)
 
             # If the content should go to a file, write it and go
             # to the next child.
@@ -179,31 +192,33 @@ class Renderable(object):
                 if func is not None:
                     val = func(StaticNode(child, val))
 
-                    # If a plain string is returned, we have no idea what 
+                    # If a plain string is returned, we have no idea what
                     # the encoding is, but we'll make a guess.
-                    if type(val) is not unicode:
+                    if type(val) is not str:
                         log.warning('The renderer for %s returned a non-unicode string.  Using the default input encoding.' % type(child).__name__)
-                        val = unicode(val, child.config['files']['input-encoding'])
+                        val = str(val)
 
                 # Write the file content
-                codecs.open(filename, 'w', 
-                            child.config['files']['output-encoding'],
-                            errors=r.encodingErrors).write(val)
+
+                enc = child.config['files']['output-encoding']
+                with open(filename, 'w', encoding=enc) as f:
+                    f.write(val)
 
                 status.info(' ] ')
 
                 continue
 
-            # Append the resultant unicode object to the output
+            # Append the resultant object to the output
             s.append(val)
 
 #       if self.filename:
 #           status.info(' ] ')
 
-        return r.outputType(u''.join(s))
+        return r.outputType(''.join(s))
+
+    # def __str__(self):
+    #     return ''
 
-    def __str__(self):
-        return unicode(self)
 
     @property
     def image(self):
@@ -223,8 +238,8 @@ class Renderable(object):
         Return the relative URL of the object
 
         If the object actually creates a file, just the filename will
-        be returned (e.g. foo.html).  If the object is within a file, 
-        both the filename and the anchor will be returned 
+        be returned (e.g. foo.html).  If the object is within a file,
+        both the filename and the anchor will be returned
         (e.g. foo.html#bar).
 
         """
@@ -234,7 +249,7 @@ class Renderable(object):
         base = self.config['document']['base-url']
         if base and base.endswith('/'):
             base = base[:-1]
-        
+
         # If this generates a file, return that filename
         if self.filename:
             if base:
@@ -263,7 +278,8 @@ class Renderable(object):
         r = Node.renderer
 
         try: return r.files[self]
-        except KeyError: pass
+        except (KeyError, TypeError): pass
+
 
         filename = None
 
@@ -279,11 +295,11 @@ class Renderable(object):
                                         r.fileExtension)
                 filename = r.files[self] = newFilename()
 
-        except AttributeError, msg:
+        except AttributeError as msg:
             if not hasattr(self, 'config'):
                 return
 
-            level = getattr(self, 'splitlevel', 
+            level = getattr(self, 'splitlevel',
                             self.config['files']['split-level'])
 
             # If our level doesn't invoke a split, don't return a filename
@@ -292,14 +308,24 @@ class Renderable(object):
 
             # Populate vars of filename generator
             # and call the generator to get the filename.
-            ns = r.newFilename.vars
+            ns = r.newFilename.variables
             if hasattr(self, 'id') and getattr(self, '@hasgenid', None) is None:
                 ns['id'] = self.id
             if hasattr(self, 'title'):
                 if hasattr(self.title, 'textContent'):
                     ns['title'] = self.title.textContent
-                elif isinstance(self.title, basestring):
+                elif isinstance(self.title, str):
                     ns['title'] = self.title
+            if hasattr(self, 'ref'):
+                ref = ''
+                if hasattr(self.ref, 'textContent'):
+                    ref = self.ref.textContent
+                elif isinstance(self.ref, str):
+                    ref = self.ref
+                if ref:
+                    ns['ref'] = ref
+            if self.nodeName:
+                ns['name'] = self.nodeName
             r.files[self] = filename = r.newFilename()
 
 #       print type(self), filename
@@ -312,12 +338,12 @@ class Renderer(dict):
     Base class for all renderers
 
     All renderers must act like a dictionary.  Each macro that is encountered
-    in a document must have a corresponding key in the renderer.  This 
+    in a document must have a corresponding key in the renderer.  This
     key points to a callable object which is called with the object to
     be rendered.
 
-    In addition to callable renderers, the renderer also handles image 
-    generation.  Images are generated when the output document type can 
+    In addition to callable renderers, the renderer also handles image
+    generation.  Images are generated when the output document type can
     not support the rendering of a macro.  One example of this is equations
     in HTML.
 
@@ -325,9 +351,9 @@ class Renderer(dict):
 
     renderableClass = Renderable
     renderMethod = None
-    textDefault = unicode
-    default = unicode
-    outputType = unicode
+    textDefault = str
+    default = str
+    outputType = str
     imageTypes = []
     vectorImageTypes = []
     fileExtension = ''
@@ -335,8 +361,8 @@ class Renderer(dict):
     imageUnits = '&${units};'
     encodingErrors = 'replace'
 
-    def __init__(self, data={}):
-        dict.__init__(self, data)
+    def __init__(self, data=None):
+        dict.__init__(self, data or {})
 
         # Names of generated files
         self.files = {}
@@ -349,8 +375,8 @@ class Renderer(dict):
         self.newFilename = None
 
     def cacheFilenames(self, node):
-        """ 
-        Generate filenames in order 
+        """
+        Generate filenames in order
 
         Since filenames are generated on demand, in order to make the
         nodes have a filename that corresponds to its position in the document,
@@ -361,7 +387,7 @@ class Renderer(dict):
 
         """
         # Using the side-effect of the filename property
-        node.filename  
+        _ = node.filename
         for child in node.childNodes:
             self.cacheFilenames(child)
 
@@ -374,14 +400,14 @@ class Renderer(dict):
 
         Required Arguments:
         document -- the document object to render
-        postProcess -- a function that will be called with the content of 
+        postProcess -- a function that will be called with the content of
 
         """
         config = document.config
 
-        # If there are no keys, print a warning.  
+        # If there are no keys, print a warning.
         # This is most likely a problem.
-        if not self.keys():
+        if not list(self.keys()):
             log.warning('There are no keys in the renderer.  ' +
                         'All objects will use the default rendering method.')
 
@@ -390,7 +416,7 @@ class Renderer(dict):
         Node.renderer = self
 
         # Create a filename generator
-        self.newFilename = Filenames(config['files'].get('filename', raw=True), 
+        self.newFilename = Filenames(config['files'].get('filename', raw=True),
                                      (config['files']['bad-chars'],
                                       config['files']['bad-chars-sub']),
                                      {'jobname':document.userdata.get('jobname', '')}, self.fileExtension)
@@ -402,20 +428,28 @@ class Renderer(dict):
         for name in names:
             if name == 'none':
                 break
-            try: 
-                exec('from plasTeX.Imagers.%s import Imager' % name)
-            except ImportError, msg:
-                log.warning("Could not load imager '%s' because '%s'" % (name, msg))
+            elif name == 'dvipng':
+                from plasTeX.Imagers.dvipng import Imager
+            elif name == 'dvi2bitmap':
+                from plasTeX.Imagers.dvi2bitmap  import Imager
+            elif name == 'pdftoppm':
+                from plasTeX.Imagers.pdftoppm  import Imager
+            elif name == 'gspdfpng':
+                from plasTeX.Imagers.gspdfpng  import Imager
+            elif name == 'gsdvipng':
+                from plasTeX.Imagers.gsdvipng  import Imager
+            elif name == 'OSXCoreGraphics':
+                from plasTeX.Imagers.OSXCoreGraphics  import Imager
+            else:
+                log.warning("Invalid imager '%s'")
                 continue
-            
+
             self.imager = Imager(document, self.imageTypes)
-    
+
             # Make sure that this imager works on this machine
             if self.imager.verify():
                 log.info('Using the imager "%s".' % name)
                 break
- 
-            self.imager = None
 
         # Still no imager? Just use the default.
         if self.imager is None:
@@ -436,19 +470,19 @@ class Renderer(dict):
         for name in names:
             if name == 'none':
                 break
-            try: 
-                exec('from plasTeX.Imagers.%s import Imager' % name)
-            except ImportError, msg:
-                log.warning("Could not load imager '%s' because '%s'" % (name, msg))
+            elif name == 'dvisvgm':
+                from plasTeX.Imagers.dvisvgm import Imager as VectorImager
+            else:
+                log.warning("Invalid imager '%s'")
                 continue
-            
-            self.vectorImager = Imager(document, self.vectorImageTypes)
-    
+
+            self.vectorImager = VectorImager(document, self.vectorImageTypes)
+
             # Make sure that this imager works on this machine
             if self.vectorImager.verify():
                 log.info('Using the vector imager "%s".' % name)
                 break
- 
+
             self.vectorImager = None
 
         # Still no vector imager? Just use the default.
@@ -470,17 +504,17 @@ class Renderer(dict):
         if type(self).renderMethod:
             getattr(document, type(self).renderMethod)()
         else:
-            unicode(document)
+            str(document)
 
         # Finish rendering images
         self.imager.close()
         self.vectorImager.close()
 
         # Run any cleanup activities
-        self.cleanup(document, self.files.values(), postProcess=postProcess)
+        self.cleanup(document, list(self.files.values()), postProcess=postProcess)
 
         # Write out auxilliary information
-        pauxname = os.path.join(document.userdata.get('working-dir','.'), 
+        pauxname = os.path.join(document.userdata.get('working-dir','.'),
                                 '%s.paux' % document.userdata.get('jobname',''))
         rname = config['general']['renderer']
         document.context.persist(pauxname, rname)
@@ -493,53 +527,54 @@ class Renderer(dict):
         return s
 
     def cleanup(self, document, files, postProcess=None):
-        """ 
-        Cleanup method called at the end of rendering 
+        """
+        Cleanup method called at the end of rendering
 
         This method allows you to do arbitrary post-processing after
         all files have been rendered.
 
-        Note: While I greatly dislike post-processing, sometimes it's 
+        Note: While I greatly dislike post-processing, sometimes it's
               just easier...
 
         Required Arguments:
         document -- the document being rendered
         files -- the list of filenames that were generated
-       
+
         Optional Arguments:
-        postProcess -- a function that will be called on the content of 
+        postProcess -- a function that will be called on the content of
             each file.  It is called with the document object and a
-            unicode object with the content of each file.  
-            It must return a unicode object.
+            string object with the content of each file.
+            It must return a string object.
 
         """
         if self.processFileContent is Renderer.processFileContent:
-            return 
+            return
 
         encoding = document.config['files']['output-encoding']
-
+        errs = self.encodingErrors
         for f in files:
             try:
-                s = codecs.open(str(f), 'r', encoding, 
-                                errors=self.encodingErrors).read()
-            except IOError, msg:
+                with open(f, 'r', encoding=encoding, errors=errs) as fd:
+                    s = fd.read()
+
+            except IOError as msg:
                 log.error(msg)
                 continue
 
             s = self.processFileContent(document, s)
-
-            if callable(postProcess):
+            if isinstance(postProcess, collections.abc.Callable):
                 s = postProcess(document, s)
 
-            codecs.open(f, 'w', encoding).write(u''.join(s))
+            with open(f, 'w', encoding=encoding) as fd:
+                fd.write(''.join(s))
 
     def find(self, keys, default=None):
         """
         Locate a renderer given a list of possibilities
-  
+
         Required Arguments:
-        keys -- a list of strings containing the requested name of 
-            a renderer.  This list is traversed in order.  The first 
+        keys -- a list of strings containing the requested name of
+            a renderer.  This list is traversed in order.  The first
             renderer that is found is returned.
 
         Keyword Arguments:
@@ -550,7 +585,7 @@ class Renderer(dict):
 
         """
         for key in keys:
-            if self.has_key(key):
+            if key in self:
                 return self[key]
 
         # Other nodes supplied default
@@ -566,15 +601,15 @@ class StaticNode(object):
     Object to assist in rendering files
 
     This object is used to wrap objects that need to have a layout
-    file wrapped around them.  The layout wrapper generally includes 
-    all of the navigation links, table of contents, etc.  
+    file wrapped around them.  The layout wrapper generally includes
+    all of the navigation links, table of contents, etc.
 
     This is simply a proxy object that returns the attributes of
-    the given object.  The exceptions are __unicode__ and __str__
-    which simply return the rendered string that was passed in.
+    the given object.  The exceptions is __str__
+    which simply returns the rendered string that was passed in.
     This allows you to use two templates: one that renders the content
-    and another that is wrapped around any node that generates a 
-    file.  Without this, you can easily run into infinite recursion 
+    and another that is wrapped around any node that generates a
+    file.  Without this, you can easily run into infinite recursion
     problems.
 
     """
@@ -583,48 +618,47 @@ class StaticNode(object):
         Initialize the static node
 
         Arguments:
-        obj -- the object that contains navigation and table of 
+        obj -- the object that contains navigation and table of
             contents information
-        content -- the rendered object in a unicode string
+        content -- the rendered object in a  string
 
         """
         self._node_data = (obj, content)
     def __getattribute__(self, name):
-        if name in ['_node_data','__unicode__','__str__']:
+        if name in ['_node_data','__str__']:
             return object.__getattribute__(self, name)
         return getattr(self._node_data[0], name)
-    def __unicode__(self):
-        return self._node_data[1]
+
     def __str__(self):
-        return unicode(self)
+        return self._node_data[1]
 
 
-class URL(unicode):
+class URL(str):
 
     def relativeTo(self, src):
         """
         Get the path of this URL relative to `src'
-    
+
         """
         if isinstance(src, Node):
             src = src.url
-    
+
         dest, src = os.path.normpath(str(self)), os.path.normpath(src)
         base = os.path.join(*(['a/'] * max(dest.count('/'), src.count('/'))))
         src = os.path.join(base, src).split('/')
         dest = os.path.join(base, dest).split('/')
-    
+
         same = 0
         for d, s in zip(dest, src):
             if d == s:
                 same += 1
                 continue
-            break 
-    
+            break
+
         dest, src = dest[same:], src[same:-1]
-    
+
         if src:
-            return type(self)(os.path.join(os.path.join(*(['..'] * len(src))), 
+            return type(self)(os.path.join(os.path.join(*(['..'] * len(src))),
                                            os.path.join(*dest)))
 
         return type(self)(os.path.join(*dest))
