@@ -5,7 +5,7 @@ TeX
 
 This module contains the facilities for parsing TeX and LaTeX source.
 The `TeX' class is an iterator interface to (La)TeX source.  Simply
-feed a `TeX' instance using the input method, then iterate over the 
+feed a `TeX' instance using the input method, then iterate over the
 expanded tokens through the standard Python iterator interface.
 
 Example:
@@ -15,14 +15,14 @@ Example:
         print token
 
 """
-
-import string, os, traceback, sys, plasTeX, codecs, subprocess, types
-from Tokenizer import Tokenizer, Token, EscapeSequence, Other
+from io import IOBase
+import string, os, sys, plasTeX, subprocess
+from plasTeX.Tokenizer import Tokenizer, Token, EscapeSequence, Other
 from plasTeX import TeXDocument
 from plasTeX.Base.TeX.Primitives import MathShift
 from plasTeX import ParameterCommand, Macro
 from plasTeX import glue, muglue, mudimen, dimen, number
-from plasTeX.Logging import getLogger, disableLogging
+from plasTeX.Logging import getLogger, disableLogging, fileLogging
 
 # Only export the TeX class
 __all__ = ['TeX']
@@ -36,11 +36,11 @@ _type = type
 class bufferediter(object):
     """ Buffered iterator """
     def __init__(self, obj):
-        self._next = iter(obj).next
+        self._next = iter(obj).__next__
         self._buffer = []
     def __iter__(self):
         return self
-    def next(self):
+    def __next__(self):
         if self._buffer:
             return self._buffer.pop()
         return self._next()
@@ -54,15 +54,20 @@ class TeX(object):
     """
     TeX Stream
 
-    This class is the central TeX engine that does all of the 
+    This class is the central TeX engine that does all of the
     parsing, invoking of macros, etc.
 
     """
     documentClass = TeXDocument
 
-    def __init__(self, ownerDocument=None, file=None):
+    def __init__(self, ownerDocument=None, myfile=None):
         if ownerDocument is None:
             ownerDocument = self.documentClass()
+            self.toplevel = True
+        elif myfile:
+            self.toplevel = True
+        else:
+            self.toplevel = False
         self.ownerDocument = ownerDocument
 
         # Input source stack
@@ -73,7 +78,7 @@ class TeX(object):
 
         # TeX arguments types and their casting functions
         self.argtypes = {
-            'url': (self.castString, {'#':12,'~':12}),
+            'url': (self.castNone, {'#':12,'~':12,'%':12,'&':12}),
             'str': self.castString,
             str: self.castString,
             'chr': self.castString,
@@ -110,21 +115,32 @@ class TeX(object):
         self.currentInput = (0,0)
 
         self.jobname = None
-        if file is not None:
-
+        if myfile is not None:
             # Filename
-            if isinstance(file, basestring):
+            if isinstance(myfile, (str, bytes)):
+                myfile = str(myfile)
+                '''
+                if config has no files structure
+                or encoding is not specified
+                or encoding is utf-8, make encoding utf_8_sig.
+                otherwise, use the specified encoding.
+                '''
                 try:
-                    encoding = self.ownerDocument.config['files']['input-encoding']
-                except:
-                    encoding = 'utf-8'
-                self.input(codecs.open(self.kpsewhich(file), 'r', encoding, 'replace'))
-                self.jobname = os.path.basename(os.path.splitext(file)[0])
+                    encoding = self.ownerDocument.config['files'].get('input-encoding', 'utf_8_sig')
+                except (KeyError, AttributeError):
+                    encoding = 'utf_8_sig'
+
+                if encoding in ['utf8', 'utf-8', 'utf_8']:
+                    encoding = 'utf_8_sig'
+
+                fname = self.kpsewhich(myfile)
+                self.input(open(fname, encoding=encoding))
+                self.jobname = os.path.basename(os.path.splitext(myfile)[0])
 
             # File object
             else:
-                self.input(file)
-                self.jobname = os.path.basename(os.path.splitext(file.name)[0])
+                self.input(myfile)
+                self.jobname = os.path.basename(os.path.splitext(myfile.name)[0])
 
     def input(self, source):
         """
@@ -138,18 +154,19 @@ class TeX(object):
         if source is None:
             return
         if self.jobname is None:
-            if isinstance(source, basestring):
+            if isinstance(source, str):
                 self.jobname = os.path.basename(os.path.splitext(source)[0])
-            elif isinstance(source, file):
+            elif isinstance(source, IOBase):
                 self.jobname = os.path.basename(os.path.splitext(source.name)[0])
+
         t = Tokenizer(source, self.ownerDocument.context)
         self.inputs.append((t, iter(t)))
         self.currentInput = self.inputs[-1]
         return self
 
     def endInput(self):
-        """ 
-        Pop the most recent input source from the stack 
+        """
+        Pop the most recent input source from the stack
 
         """
         if self.inputs:
@@ -157,22 +174,23 @@ class TeX(object):
         if self.inputs:
             self.currentInput = self.inputs[-1]
 
-    def loadPackage(self, file, options={}):
+    def loadPackage(self, myfile, options=None):
         """
         Load a LaTeX package
 
         Required Arguments:
-        file -- name of the file to load
+        myfile -- name of the file to load
 
         Keyword Arguments:
         options -- options passed to the macro which is loading the package
 
         """
+        options = options or {}
         config = self.ownerDocument.config
 
         try:
-            path = self.kpsewhich(file)
-        except OSError, msg:
+            path = self.kpsewhich(myfile)
+        except OSError as msg:
             log.warning(msg)
             return False
 
@@ -181,24 +199,23 @@ class TeX(object):
 
         try:
             encoding = config['files']['input-encoding']
-            f = codecs.open(path, 'r', encoding, 'replace')
-            # Put in a flag so that we can parse past our own
-            # package tokens and throw them away, we don't want them in
-            # the output document.
-            flag = plasTeX.Command()
-            self.pushToken(flag)
-            encoding = config['files']['input-encoding']
-            self.input(f)
-            self.ownerDocument.context.packages[file] = options or {}
-            for tok in self:
-                if tok is flag:
-                    break
+            with open(path, 'r', encoding=encoding) as f:
+                # Put in a flag so that we can parse past our own
+                # package tokens and throw them away, we don't want them in
+                # the output document.
+                flag = plasTeX.Command()
+                self.pushToken(flag)
+                self.input(f)
+                self.ownerDocument.context.packages[myfile] = options or {}
+                for tok in self:
+                    if tok is flag:
+                        break
 
-        except (OSError, IOError, TypeError), msg:
+        except (OSError, IOError, TypeError) as msg:
             if msg:
-                msg = ' ( %s )' % msg
+                msg = ' ( %s )' % str(msg)
             # Failed to load LaTeX style file
-            log.warning('Error opening package "%s"%s', file, msg)
+            log.warning('Error opening package "%s"%s', myfile, msg)
             status.info(' ) ')
             return False
 
@@ -223,8 +240,12 @@ class TeX(object):
         """ Turn off logging """
         disableLogging()
 
+    def fileLogging(self):
+        fname = '%s/%s.log' % (os.path.dirname(self.filename), self.jobname)
+        fileLogging(fname)
+
     def itertokens(self):
-        """ 
+        """
         Iterate over unexpanded tokens
 
         Returns:
@@ -241,7 +262,7 @@ class TeX(object):
             # Always get next token from top of input stack
             try:
                 while 1:
-                    t = inputs[-1][-1].next()
+                    t = next(inputs[-1][-1])
                     # Save context depth of each token for use in digestion
                     t.contextDepth = context.depth
                     t.ownerDocument = ownerDocument
@@ -255,16 +276,41 @@ class TeX(object):
             except IndexError:
                 break
 
+    def iterchars(self):
+        """
+        Iterate over input characters (untokenized)
+
+        Returns:
+        generator that iterates through the untokenized characters
+
+        """
+        # Create locals before going into generator loop
+        inputs = self.inputs
+        context = self.ownerDocument.context
+        endInput = self.endInput
+        ownerDocument = self.ownerDocument
+
+        while inputs:
+            # Walk through characters
+            try:
+                for char in inputs[-1][0].iterchars():
+                    yield char
+                else:
+                    endInput()
+            # This really shouldn't happen, but just in case...
+            except IndexError:
+                break
+
     def __iter__(self):
-        """ 
-        Iterate over tokens while expanding them 
+        """
+        Iterate over tokens while expanding them
 
         Returns:
         generator that iterates through the expanded tokens
 
         """
         # Cache variables before starting the generator
-        next = self.itertokens().next
+        next = self.itertokens().__next__
         pushToken = self.pushToken
         pushTokens = self.pushTokens
         createElement = self.ownerDocument.createElement
@@ -272,12 +318,15 @@ class TeX(object):
 
         while 1:
             # Get the next token
-            token = next()
+            try:
+                token = next()
+            except StopIteration:
+                return
 
             # Token is null, ignore it
             if token is None:
                 continue
-                
+
             # Macro that has already been expanded
             elif token.nodeType == ELEMENT_NODE:
                 pass
@@ -301,9 +350,10 @@ class TeX(object):
 #                       log.info('expanding %s %s', token.macroName, ''.join([x.source for x in tokens]))
                         pushTokens(tokens)
                     continue
-                except Exception, msg:
-                    if str(msg).strip():
-                        msg = ' (%s)' % str(msg).strip()
+                except Exception as message:
+                    msg = str(message)
+                    if msg.strip():
+                        msg = ' (%s)' % msg.strip()
                     log.error('Error while expanding "%s"%s%s',
                               token.macroName, self.lineInfo, msg)
                     raise
@@ -367,8 +417,8 @@ class TeX(object):
 
 
     def parse(self, output=None):
-        """ 
-        Parse stream content until it is empty 
+        """
+        Parse stream content until it is empty
 
         Keyword Arguments:
         output -- object to put the content in.  This should be either
@@ -389,12 +439,16 @@ class TeX(object):
                     item.parentNode = output
                     item.digest(tokens)
                 output.append(item)
-        except Exception, msg:
-            if str(msg).strip():
-               msg = ' (%s)' % str(msg).strip()
+        except Exception as message:
+            msg = str(message)
+            if msg.strip():
+               msg = ' (%s)' % msg.strip()
             log.error('An error occurred while building the document object%s%s', self.lineInfo, msg)
             raise
 
+        if self.toplevel:
+            for callback in self.ownerDocument.postParseCallbacks:
+                callback()
         return output
 
     def textTokens(self, text):
@@ -405,7 +459,7 @@ class TeX(object):
         text -- string containing text to be tokenized
 
         """
-        return [Other(x) for x in unicode(text)]
+        return [Other(x) for x in text]
 
     def pushToken(self, token):
         """
@@ -448,7 +502,7 @@ class TeX(object):
         string containing the TeX source
 
         """
-        return u''.join([x.source for x in tokens])
+        return ''.join([x.source for x in tokens])
 
     def normalize(self, tokens):
         """
@@ -467,15 +521,15 @@ class TeX(object):
             return tokens
 
         grouptokens = [Token.CC_EGROUP, Token.CC_BGROUP]
-        textTokens = [Token.CC_LETTER, Token.CC_OTHER, 
-                      Token.CC_EGROUP, Token.CC_BGROUP, 
+        textTokens = [Token.CC_LETTER, Token.CC_OTHER,
+                      Token.CC_EGROUP, Token.CC_BGROUP,
                       Token.CC_SPACE]
 
         try: iter(tokens)
         except TypeError: return tokens
 
         for t in tokens:
-            if isinstance(t, basestring):
+            if isinstance(t, str):
                 continue
             # Element nodes can't be part of normalized text
             if t.nodeType == Macro.ELEMENT_NODE:
@@ -495,23 +549,20 @@ class TeX(object):
                 t.normalize(getattr(self.ownerDocument, 'charsubs', []))
                 return t
 
-        return (u''.join([x for x in tokens 
-                          if getattr(x, 'catcode', Token.CC_OTHER) 
+        return (''.join([x for x in tokens
+                          if getattr(x, 'catcode', Token.CC_OTHER)
                              not in grouptokens])).strip()
 
-    def readIfContent(self, which, debug=False):
+    def processIfContent(self, which, debug=False):
         """
-        Return the requested portion of the `if' block
+        Process the requested portion of the `if' block
 
         Required Arguments:
-        which -- the case to return.  If this is a boolean, a value of 
+        which -- the case to return.  If this is a boolean, a value of
             `True' will return the first part of the `if' block.  If it
-            is `False', it will return the `else' portion.  If this is 
+            is `False', it will return the `else' portion.  If this is
             an integer, the `case' matching this integer will be returned.
 
-        Returns:
-        list of tokens from the requested portion of the `if' block
-
         """
         # Since the true content always comes first, we need to set
         # True to case 0 and False to case 1.
@@ -519,37 +570,35 @@ class TeX(object):
         if isinstance(which, bool):
             if which: which = 0
             else: which = 1
+
         cases = [[]]
         nesting = 0
+
         for t in self.itertokens():
-            # This is probably going to cause some trouble, there 
-            # are bound to be macros that start with 'if' that aren't
-            # actually 'if' constructs...
-            if t.nodeType == Macro.ELEMENT_NODE:
-                if t.nodeName.startswith('if'):
-                    nesting += 1
-            elif t.startswith('if'):
+            name = getattr(t, 'macroName', '') or ''
+            if name.startswith('if'):
+                cases[-1].append(t)
                 nesting += 1
-            elif t == 'fi':
-                if not nesting:
-                    self.readOptionalSpaces()
+            elif name == 'fi':
+                if nesting > 1:
+                    cases[-1].append(t)
+                elif not nesting:
                     break
                 nesting -= 1
-            elif not(nesting) and t == 'else':
-                self.readOptionalSpaces()
-                cases.append([])
-                elsefound = True
-                continue
-            elif not(nesting) and t == 'or':
-                self.readOptionalSpaces()
+            elif not(nesting) and name == 'else':
                 cases.append([])
                 continue
-            cases[-1].append(t)
-        if debug:
-            print 'CASES', cases
-        if not elsefound:
-            cases.append([])
-        return cases[which]
+            elif not(nesting) and name == 'or':
+                cases.append([])
+                continue
+            else:
+                cases[-1].append(t)
+
+        # else case for ifs without elses
+        cases.append([])
+
+        # Push if-selected tokens back into tokenizer
+        self.pushTokens(cases[which])
 
     def readArgument(self, *args, **kwargs):
         """
@@ -561,10 +610,10 @@ class TeX(object):
         """
         return self.readArgumentAndSource(*args, **kwargs)[0]
 
-    def readArgumentAndSource(self, spec=None, type=None, subtype=None, 
+    def readArgumentAndSource(self, spec=None, type=None, subtype=None,
                     delim=',', expanded=False, default=None, parentNode=None,
                     name=None, stripLeadingWhitespace=True):
-        """ 
+        """
         Get an argument and the TeX source that created it
 
         Optional Arguments:
@@ -573,7 +622,7 @@ class TeX(object):
             returned.  If it is a two-character string, a grouping
             delimited by those two characters is returned (i.e. '[]').
             If it is a single-character string, the stream is checked
-            to see if the next character is the one specified.  
+            to see if the next character is the one specified.
             In all cases, if the specified argument is not found,
             'None' is returned.
         type -- data type to cast the argument to.  New types can be
@@ -585,7 +634,7 @@ class TeX(object):
         subtype -- data type to use for elements of a list or dictionary
         delim -- item delimiter for list and dictionary types
         expanded -- boolean indicating whether the argument content
-            should be expanded or just returned as an unexpanded 
+            should be expanded or just returned as an unexpanded
             text string
         default -- value to return if the argument doesn't exist
         parentNode -- the node that the argument belongs to
@@ -600,7 +649,7 @@ class TeX(object):
         object of type `type` -- if `type` was specified
         list of tokens -- for all other arguments
 
-        The second argument is a string containing the TeX source 
+        The second argument is a string containing the TeX source
         for the argument.
 
         """
@@ -667,7 +716,7 @@ class TeX(object):
                     self.pushToken(t)
                     break
                 else:
-                    args.append(t) 
+                    args.append(t)
             else: pass
             ParameterCommand.enable()
             return args, self.source(args)
@@ -678,7 +727,7 @@ class TeX(object):
                 if t is None or t == '':
                     continue
                 if t.catcode == Token.CC_SPACE:
-                    break 
+                    break
                 toks.append(t)
             return self.expandTokens(toks, parentNode=parentNode), self.source(toks)
 
@@ -690,8 +739,8 @@ class TeX(object):
         try:
             # Set catcodes for this argument type
             try:
-                if isinstance(self.argtypes[type], (list,tuple)): 
-                    for key, value in self.argtypes[type][1].items():
+                if isinstance(self.argtypes[type], (list,tuple)):
+                    for key, value in list(self.argtypes[type][1].items()):
                         priorcodes[key] = self.ownerDocument.context.whichCode(key)
                         self.ownerDocument.context.catcode(key, value)
             except KeyError:
@@ -704,28 +753,27 @@ class TeX(object):
             # Get a single character argument
             elif len(spec) == 1:
                 toks, source = self.readCharacter(spec)
-    
+
             # Get an argument grouped by the given characters (e.g. [...], (...))
             elif len(spec) == 2:
                 toks, source = self.readGrouping(spec, expanded, parentNode=parentNode)
-    
+
             # This isn't a correct value
             else:
-                raise ValueError, 'Unrecognized specifier "%s"' % spec
+                raise ValueError('Unrecognized specifier "%s"' % spec)
 
-        except Exception, msg:
+        except Exception as msg:
             log.error('Error while reading argument "%s" of %s%s (%s)' % \
                           (name, parentNode.nodeName, self.lineInfo, msg))
-            raise 
+            raise
 
         # Set catcodes back to original values
-        for key, value in priorcodes.items():
+        for key, value in list(priorcodes.items()):
             self.ownerDocument.context.catcode(key, value)
 
         if toks is None:
             ParameterCommand.enable()
             return default, ''
-
         res = self.cast(toks, type, subtype, delim, parentNode, name)
 
         # Normalize any document fragments
@@ -789,7 +837,7 @@ class TeX(object):
                 toks = self.expandTokens(toks, parentNode=parentNode)
                 if isgroup:
                     s = self.source(toks)
-                    source = u'%s%s%s' % (source[0].source, s, 
+                    source = '%s%s%s' % (source[0].source, s,
                                           source[-1].source)
                 else:
                     source = self.source(toks)
@@ -840,16 +888,16 @@ class TeX(object):
             source = [t]
             # A [ ... ], ( ... ), etc. grouping was found
             if t.catcode != Token.CC_ESCAPE and \
-               (t == begin or unicode(t) == unicode(begin)):
+               (t == begin or str(t) == str(begin)):
                 level = 1
                 for t in tokens:
                     source.append(t)
                     if t.catcode != Token.CC_ESCAPE and \
-                       (t == begin or unicode(t) == unicode(begin)):
+                       (t == begin or str(t) == str(begin)):
                         toks.append(t)
                         level += 1
                     elif t.catcode != Token.CC_ESCAPE and \
-                         (t == end or unicode(t) == unicode(end)):
+                         (t == end or str(t) == str(end)):
                         level -= 1
                         if level == 0:
                             break
@@ -895,14 +943,14 @@ class TeX(object):
 
         return result
 
-    def cast(self, tokens, dtype, subtype=None, delim=',', 
+    def cast(self, tokens, dtype, subtype=None, delim=',',
                    parentNode=None, name=None):
-        """ 
+        """
         Cast the tokens to the appropriate type
 
         This method is used to convert tokens into Python objects.
         This happens when the user has specified that a macro argument
-        should be a dictionary (e.g. foo:dict), 
+        should be a dictionary (e.g. foo:dict),
         a list (e.g. foo:list), etc.
 
         Required Arguments:
@@ -918,7 +966,7 @@ class TeX(object):
 
         """
         argtypes = {}
-        for key, t in self.argtypes.items():
+        for key, t in list(self.argtypes.items()):
             if isinstance(t, tuple):
                 argtypes[key] = t[0]
             else:
@@ -929,21 +977,23 @@ class TeX(object):
             pass
 
         # Could not find specified type
-        elif not argtypes.has_key(dtype):
+        elif dtype not in list(argtypes.keys()):
             log.warning('Could not find datatype "%s"' % dtype)
-            pass
 
         # Casting to specified type
         else:
-            tokens = argtypes[dtype](tokens, subtype=subtype, 
+            tokens = argtypes[dtype](tokens, subtype=subtype,
                      delim=delim, parentNode=parentNode, name=name)
 
-        # Set parent node as needed 
+        # Set parent node as needed
         if getattr(tokens,'nodeType',None) == Macro.DOCUMENT_FRAGMENT_NODE:
             tokens.parentNode = parentNode
 
         return tokens
 
+    def castNone(self, tokens, **kwargs):
+        return tokens
+
     def castControlSequence(self, tokens, **kwargs):
         """
         Limit the argument to a single non-space token
@@ -956,9 +1006,10 @@ class TeX(object):
         self.cast()
 
         """
+
         return [x for x in tokens if x.catcode == Token.CC_ESCAPE].pop(0)
 
-    def castString(self, tokens, type=unicode, **kwargs):
+    def castString(self, tokens, type=str, **kwargs):
         """
         Join the tokens into a string
 
@@ -1017,7 +1068,7 @@ class TeX(object):
         ref = self.castString(tokens, **kwargs)
         self.ownerDocument.context.ref(kwargs['parentNode'], kwargs['name'], ref)
         return ref
-        
+
     def castNumber(self, tokens, **kwargs):
         """
         Join the tokens into a string and turn the result into an integer
@@ -1039,7 +1090,7 @@ class TeX(object):
 #       try: return number(self.castString(tokens, **kwargs))
 #       except: return number(0)
         return self.readInternalType(tokens, self.readNumber)
-        
+
     def castDecimal(self, tokens, **kwargs):
         """
         Join the tokens into a string and turn the result into a float
@@ -1118,7 +1169,7 @@ class TeX(object):
 #       try: return glue(self.castString(tokens, **kwargs))
 #       except: return glue(0)
         return self.readInternalType(tokens, self.readGlue)
- 
+
     def castMuGlue(self, tokens, **kwargs):
         """
         Jain the tokens into a string and convert the result into a `MuGlue`
@@ -1137,7 +1188,7 @@ class TeX(object):
 #       try: return muglue(self.castString(tokens, **kwargs))
 #       except: return muglue(0)
         return self.readInternalType(tokens, self.readMuGlue)
-                             
+
     def castList(self, tokens, type=list, **kwargs):
         """
         Parse items delimited by the given delimiter into a list
@@ -1186,7 +1237,7 @@ class TeX(object):
                 listarg[-1].append(current)
 
             else:
-                listarg[-1].append(current) 
+                listarg[-1].append(current)
 
         return type([self.normalize(self.cast(x, subtype)) for x in listarg])
 
@@ -1209,7 +1260,7 @@ class TeX(object):
         self.readArgument()
         self.cast()
 
-        """      
+        """
         delim = kwargs.get('delim')
         if delim is None:
             delim = ','
@@ -1277,7 +1328,7 @@ class TeX(object):
         return dictarg
 
     def kpsewhich(self, name):
-        """ 
+        """
         Locate the given file using kpsewhich
 
         Required Arguments:
@@ -1297,22 +1348,36 @@ class TeX(object):
             # I think this happens only for the command line file.
             pass
         else:
-            TEXINPUTS = os.environ["TEXINPUTS"]
-            os.environ["TEXINPUTS"] = "%s:%s" % (srcDir, TEXINPUTS)
+            TEXINPUTS = os.environ.get("TEXINPUTS",'')
+            os.environ["TEXINPUTS"] = "%s%s%s%s" % (srcDir, os.path.pathsep, TEXINPUTS, os.path.pathsep)
 
         try:
             program = self.ownerDocument.config['general']['kpsewhich']
-            output = subprocess.Popen([program, name], 
-                                  stdout=subprocess.PIPE).communicate()[0].strip()
+
+            kwargs = {'stdout':subprocess.PIPE}
+            if sys.platform.lower().startswith('win'):
+                kwargs['shell'] = True
+
+            output = subprocess.Popen([program, name], **kwargs).communicate()[0].strip()
+            output = output.decode('utf-8')
             if output:
                 return output
 
-            raise OSError, 'Could not find any file named: %s' % name
+        except:
+            fullname = ''
+            paths = os.environ.get("TEXINPUTS", '.').split(os.path.pathsep)
+            for path in [x for x in paths if x]:
+                if name in os.listdir(path):
+                    fullname = os.path.join(path,name)
+                    break
+            if fullname:
+                return fullname
 
-        finally:
-            # Undo any mods to $TEXINPUTS.
-            if TEXINPUTS is not None:
-                os.environ["TEXINPUTS"] = TEXINPUTS
+        # Undo any mods to $TEXINPUTS.
+        if TEXINPUTS:
+            os.environ["TEXINPUTS"] = TEXINPUTS
+
+        raise OSError('Could not find any file named: %s' % name)
 
 #
 # Parsing helper methods for parsing numbers, spaces, dimens, etc.
@@ -1329,12 +1394,12 @@ class TeX(object):
                 continue
             elif t.catcode != Token.CC_SPACE:
                 self.pushToken(t)
-                break 
+                break
             tokens.append(t)
         return tokens
 
     def readKeyword(self, words, optspace=True):
-        """ 
+        """
         Read keyword from the stream
 
         Required Arguments:
@@ -1360,7 +1425,7 @@ class TeX(object):
                 if t.upper() == letters[0]:
                     letters.pop(0)
                     if not letters:
-                        if optspace: 
+                        if optspace:
                             self.readOneOptionalSpace()
                         return word
                 else:
@@ -1449,18 +1514,18 @@ class TeX(object):
         true = self.readKeyword(['true'])
         unit = self.readKeyword(units)
         if unit is None:
-            log.warning('Missing unit (expecting %s)%s, treating as `%s`', 
+            log.warning('Missing unit (expecting %s)%s, treating as `%s`',
                         ', '.join(units), self.lineInfo, units[0])
             unit = units[0]
         ParameterCommand.enable()
         return dimen('1%s' % unit)
 
     def readOptionalSigns(self):
-        """ 
-        Read optional + and - signs 
+        """
+        Read optional + and - signs
 
         Returns:
-        +1 or -1 
+        +1 or -1
 
         """
         sign = 1
@@ -1499,11 +1564,11 @@ class TeX(object):
 
         Required Arguments:
         chars -- sequence of characters that should be accepted
-        
+
         Keyword Arguments:
-        optspace -- boolean indicating if an optional space should 
+        optspace -- boolean indicating if an optional space should
             be absorbed after the sequence of characters
-        default -- string to return if none of the characters in 
+        default -- string to return if none of the characters in
             the given set are found
 
         Returns:
@@ -1515,7 +1580,7 @@ class TeX(object):
         for t in self:
             if t.nodeType == Macro.ELEMENT_NODE:
                 self.pushToken(t)
-                break 
+                break
             if t not in chars:
                 if optspace and t.catcode == Token.CC_SPACE:
                     pass
@@ -1525,9 +1590,9 @@ class TeX(object):
             output.append(t)
         if not output:
             return default
-        return u''.join(output)
+        return ''.join(output)
 
-    def readInteger(self):
+    def readInteger(self, optspace=True):
         """
         Read an integer from the stream
 
@@ -1538,6 +1603,7 @@ class TeX(object):
         ParameterCommand.disable()
         num = None
         sign = self.readOptionalSigns()
+
         for t in self:
             # internal/coerced integers
             if t.nodeType == Macro.ELEMENT_NODE:
@@ -1548,7 +1614,8 @@ class TeX(object):
                     break
             # integer constant
             elif t in string.digits:
-                num = number(sign * int(t + self.readSequence(string.digits)))
+                num = number(sign * int(t + self.readSequence(string.digits,
+                                                              optspace=optspace)))
                 for t in self:
                     if t.nodeType == Macro.ELEMENT_NODE and \
                        isinstance(t, ParameterCommand):
@@ -1558,10 +1625,12 @@ class TeX(object):
                     break
             # octal constant
             elif t == "'":
-                num = number(sign * int('0' + self.readSequence(string.octdigits, default='0'), 8))
+                num = number(sign * int('0' + self.readSequence(string.octdigits,
+                                                   default='0', optspace=optspace), 8))
             # hex constant
             elif t == '"':
-                num = number(sign * int('0x' + self.readSequence(string.hexdigits, default='0'), 16))
+                num = number(sign * int('0x' + self.readSequence(string.hexdigits,
+                                               default='0', optspace=optspace), 16))
             # character token
             elif t == '`':
                 for t in self.itertokens():
@@ -1599,7 +1668,7 @@ class TeX(object):
         if self.readKeyword(['plus']):
             return self.readDimen(units=dimen.units+['filll','fill','fil'])
         return None
-            
+
     def readShrink(self):
         """ Read a shrink parameter from the stream """
         if self.readKeyword(['minus']):
@@ -1629,7 +1698,7 @@ class TeX(object):
         if self.readKeyword(['plus']):
             return self.readDimen(units=mudimen.units+['filll','fill','fil'])
         return None
-            
+
     def readMuShrink(self):
         """ Read a mushrink parameter from the stream """
         if self.readKeyword(['minus']):
@@ -1651,7 +1720,7 @@ class TeX(object):
             for item in self:
                 if item is dummy:
                     break
-        except OSError, msg:
+        except OSError as msg:
             log.warning(msg)
         self.ownerDocument.context.warnOnUnrecognized = warn
 
@@ -1660,4 +1729,3 @@ class TeX(object):
 #       """ Return the basename of the main input file """
 #       print self.inputs
 #       return os.path.basename(os.path.splitext(self.inputs[0][0].filename)[0])
-
